@@ -1,13 +1,11 @@
 """Support for Climate devices of (EMEA/EU-based) Honeywell TCC systems."""
+from __future__ import annotations
+
 from datetime import datetime as dt
 import logging
-from typing import List, Optional
 
-from homeassistant.components.climate import ClimateDevice
+from homeassistant.components.climate import ClimateEntity
 from homeassistant.components.climate.const import (
-    CURRENT_HVAC_HEAT,
-    CURRENT_HVAC_IDLE,
-    CURRENT_HVAC_OFF,
     HVAC_MODE_AUTO,
     HVAC_MODE_HEAT,
     HVAC_MODE_OFF,
@@ -19,8 +17,9 @@ from homeassistant.components.climate.const import (
     SUPPORT_TARGET_TEMPERATURE,
 )
 from homeassistant.const import PRECISION_TENTHS
-from homeassistant.helpers.typing import ConfigType, HomeAssistantType
-from homeassistant.util.dt import parse_datetime
+from homeassistant.core import HomeAssistant
+from homeassistant.helpers.typing import ConfigType
+import homeassistant.util.dt as dt_util
 
 from . import (
     ATTR_DURATION_DAYS,
@@ -77,7 +76,7 @@ STATE_ATTRS_ZONES = ["zoneId", "activeFaults", "setpointStatus", "temperatureSta
 
 
 async def async_setup_platform(
-    hass: HomeAssistantType, config: ConfigType, async_add_entities, discovery_info=None
+    hass: HomeAssistant, config: ConfigType, async_add_entities, discovery_info=None
 ) -> None:
     """Create the evohome Controller, and its Zones, if any."""
     if discovery_info is None:
@@ -122,7 +121,7 @@ async def async_setup_platform(
     async_add_entities([controller] + zones, update_before_add=True)
 
 
-class EvoClimateDevice(EvoDevice, ClimateDevice):
+class EvoClimateEntity(EvoDevice, ClimateEntity):
     """Base for an evohome Climate device."""
 
     def __init__(self, evo_broker, evo_device) -> None:
@@ -132,24 +131,29 @@ class EvoClimateDevice(EvoDevice, ClimateDevice):
         self._preset_modes = None
 
     @property
-    def hvac_modes(self) -> List[str]:
+    def hvac_modes(self) -> list[str]:
         """Return a list of available hvac operation modes."""
         return list(HA_HVAC_TO_TCS)
 
     @property
-    def preset_modes(self) -> Optional[List[str]]:
+    def preset_modes(self) -> list[str] | None:
         """Return a list of available preset modes."""
         return self._preset_modes
 
 
-class EvoZone(EvoChild, EvoClimateDevice):
+class EvoZone(EvoChild, EvoClimateEntity):
     """Base for a Honeywell TCC Zone."""
 
     def __init__(self, evo_broker, evo_device) -> None:
         """Initialize a Honeywell TCC Zone."""
         super().__init__(evo_broker, evo_device)
 
-        self._unique_id = evo_device.zoneId
+        if evo_device.modelType.startswith("VisionProWifi"):
+            # this system does not have a distinct ID for the zone
+            self._unique_id = f"{evo_device.zoneId}z"
+        else:
+            self._unique_id = evo_device.zoneId
+
         self._name = evo_device.name
         self._icon = "mdi:radiator"
 
@@ -170,21 +174,21 @@ class EvoZone(EvoChild, EvoClimateDevice):
             return
 
         # otherwise it is SVC_SET_ZONE_OVERRIDE
-        temp = round(data[ATTR_ZONE_TEMP] * self.precision) / self.precision
-        temp = max(min(temp, self.max_temp), self.min_temp)
+        temperature = max(min(data[ATTR_ZONE_TEMP], self.max_temp), self.min_temp)
 
         if ATTR_DURATION_UNTIL in data:
             duration = data[ATTR_DURATION_UNTIL]
             if duration.total_seconds() == 0:
                 await self._update_schedule()
-                until = parse_datetime(str(self.setpoints.get("next_sp_from")))
+                until = dt_util.parse_datetime(self.setpoints.get("next_sp_from", ""))
             else:
-                until = dt.now() + data[ATTR_DURATION_UNTIL]
+                until = dt_util.now() + data[ATTR_DURATION_UNTIL]
         else:
             until = None  # indefinitely
 
+        until = dt_util.as_utc(until) if until else None
         await self._evo_broker.call_client_api(
-            self._evo_device.set_temperature(temperature=temp, until=until)
+            self._evo_device.set_temperature(temperature, until=until)
         )
 
     @property
@@ -195,26 +199,13 @@ class EvoZone(EvoChild, EvoClimateDevice):
         is_off = self.target_temperature <= self.min_temp
         return HVAC_MODE_OFF if is_off else HVAC_MODE_HEAT
 
-    @property
-    def hvac_action(self) -> Optional[str]:
-        """Return the current running hvac operation if supported."""
-        if self._evo_tcs.systemModeStatus["mode"] == EVO_HEATOFF:
-            return CURRENT_HVAC_OFF
-        if self.target_temperature <= self.min_temp:
-            return CURRENT_HVAC_OFF
-        if not self._evo_device.temperatureStatus["isAvailable"]:
-            return None
-        if self.target_temperature <= self.current_temperature:
-            return CURRENT_HVAC_IDLE
-        return CURRENT_HVAC_HEAT
-
     @property
     def target_temperature(self) -> float:
         """Return the target temperature of a Zone."""
         return self._evo_device.setpointStatus["targetHeatTemperature"]
 
     @property
-    def preset_mode(self) -> Optional[str]:
+    def preset_mode(self) -> str | None:
         """Return the current preset mode, e.g., home, away, temp."""
         if self._evo_tcs.systemModeStatus["mode"] in [EVO_AWAY, EVO_HEATOFF]:
             return TCS_PRESET_TO_HA.get(self._evo_tcs.systemModeStatus["mode"])
@@ -244,12 +235,13 @@ class EvoZone(EvoChild, EvoClimateDevice):
         if until is None:
             if self._evo_device.setpointStatus["setpointMode"] == EVO_FOLLOW:
                 await self._update_schedule()
-                until = parse_datetime(str(self.setpoints.get("next_sp_from")))
+                until = dt_util.parse_datetime(self.setpoints.get("next_sp_from", ""))
             elif self._evo_device.setpointStatus["setpointMode"] == EVO_TEMPOVER:
-                until = parse_datetime(self._evo_device.setpointStatus["until"])
+                until = dt_util.parse_datetime(self._evo_device.setpointStatus["until"])
 
+        until = dt_util.as_utc(until) if until else None
         await self._evo_broker.call_client_api(
-            self._evo_device.set_temperature(temperature, until)
+            self._evo_device.set_temperature(temperature, until=until)
         )
 
     async def async_set_hvac_mode(self, hvac_mode: str) -> None:
@@ -278,7 +270,7 @@ class EvoZone(EvoChild, EvoClimateDevice):
                 self._evo_device.cancel_temp_override()
             )
 
-    async def async_set_preset_mode(self, preset_mode: Optional[str]) -> None:
+    async def async_set_preset_mode(self, preset_mode: str | None) -> None:
         """Set the preset mode; if None, then revert to following the schedule."""
         evo_preset_mode = HA_PRESET_TO_EVO.get(preset_mode, EVO_FOLLOW)
 
@@ -292,12 +284,13 @@ class EvoZone(EvoChild, EvoClimateDevice):
 
         if evo_preset_mode == EVO_TEMPOVER:
             await self._update_schedule()
-            until = parse_datetime(str(self.setpoints.get("next_sp_from")))
+            until = dt_util.parse_datetime(self.setpoints.get("next_sp_from", ""))
         else:  # EVO_PERMOVER
             until = None
 
+        until = dt_util.as_utc(until) if until else None
         await self._evo_broker.call_client_api(
-            self._evo_device.set_temperature(temperature, until)
+            self._evo_device.set_temperature(temperature, until=until)
         )
 
     async def async_update(self) -> None:
@@ -308,7 +301,7 @@ class EvoZone(EvoChild, EvoClimateDevice):
             self._device_state_attrs[attr] = getattr(self._evo_device, attr)
 
 
-class EvoController(EvoClimateDevice):
+class EvoController(EvoClimateEntity):
     """Base for a Honeywell TCC Controller/Location.
 
     The Controller (aka TCS, temperature control system) is the parent of all the child
@@ -345,20 +338,23 @@ class EvoController(EvoClimateDevice):
             mode = EVO_RESET
 
         if ATTR_DURATION_DAYS in data:
-            until = dt.combine(dt.now().date(), dt.min.time())
+            until = dt_util.start_of_local_day()
             until += data[ATTR_DURATION_DAYS]
 
         elif ATTR_DURATION_HOURS in data:
-            until = dt.now() + data[ATTR_DURATION_HOURS]
+            until = dt_util.now() + data[ATTR_DURATION_HOURS]
 
         else:
             until = None
 
         await self._set_tcs_mode(mode, until=until)
 
-    async def _set_tcs_mode(self, mode: str, until: Optional[dt] = None) -> None:
+    async def _set_tcs_mode(self, mode: str, until: dt | None = None) -> None:
         """Set a Controller to any of its native EVO_* operating modes."""
-        await self._evo_broker.call_client_api(self._evo_tcs.set_status(mode))
+        until = dt_util.as_utc(until) if until else None
+        await self._evo_broker.call_client_api(
+            self._evo_tcs.set_status(mode, until=until)
+        )
 
     @property
     def hvac_mode(self) -> str:
@@ -367,7 +363,7 @@ class EvoController(EvoClimateDevice):
         return HVAC_MODE_OFF if tcs_mode == EVO_HEATOFF else HVAC_MODE_HEAT
 
     @property
-    def current_temperature(self) -> Optional[float]:
+    def current_temperature(self) -> float | None:
         """Return the average current temperature of the heating Zones.
 
         Controllers do not have a current temp, but one is expected by HA.
@@ -380,7 +376,7 @@ class EvoController(EvoClimateDevice):
         return round(sum(temps) / len(temps), 1) if temps else None
 
     @property
-    def preset_mode(self) -> Optional[str]:
+    def preset_mode(self) -> str | None:
         """Return the current preset mode, e.g., home, away, temp."""
         return TCS_PRESET_TO_HA.get(self._evo_tcs.systemModeStatus["mode"])
 
@@ -402,7 +398,7 @@ class EvoController(EvoClimateDevice):
         """Set an operating mode for a Controller."""
         await self._set_tcs_mode(HA_HVAC_TO_TCS.get(hvac_mode))
 
-    async def async_set_preset_mode(self, preset_mode: Optional[str]) -> None:
+    async def async_set_preset_mode(self, preset_mode: str | None) -> None:
         """Set the preset mode; if None, then revert to 'Auto' mode."""
         await self._set_tcs_mode(HA_PRESET_TO_TCS.get(preset_mode, EVO_AUTO))
 
