@@ -9,6 +9,7 @@ from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import DATA_RATE_KILOBYTES_PER_SECOND
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers.dispatcher import async_dispatcher_connect
+from homeassistant.helpers.entity import DeviceInfo
 import homeassistant.util.dt as dt_util
 
 from .const import (
@@ -49,25 +50,23 @@ async def async_setup_entry(
             )
         )
 
-    for sensor_key in CONNECTION_SENSORS:
-        entities.append(
-            FreeboxSensor(router, sensor_key, CONNECTION_SENSORS[sensor_key])
-        )
+    for sensor_key, sensor in CONNECTION_SENSORS.items():
+        entities.append(FreeboxSensor(router, sensor_key, sensor))
 
-    for sensor_key in CALL_SENSORS:
-        entities.append(FreeboxCallSensor(router, sensor_key, CALL_SENSORS[sensor_key]))
+    for sensor_key, sensor in CALL_SENSORS.items():
+        entities.append(FreeboxCallSensor(router, sensor_key, sensor))
 
     _LOGGER.debug("%s - %s - %s disk(s)", router.name, router.mac, len(router.disks))
     for disk in router.disks.values():
         for partition in disk["partitions"]:
-            for sensor_key in DISK_PARTITION_SENSORS:
+            for sensor_key, sensor in DISK_PARTITION_SENSORS.items():
                 entities.append(
                     FreeboxDiskSensor(
                         router,
                         disk,
                         partition,
                         sensor_key,
-                        DISK_PARTITION_SENSORS[sensor_key],
+                        sensor,
                     )
                 )
 
@@ -130,7 +129,7 @@ class FreeboxSensor(SensorEntity):
         return self._device_class
 
     @property
-    def device_info(self) -> dict[str, Any]:
+    def device_info(self) -> DeviceInfo:
         """Return the device information."""
         return self._router.device_info
 
@@ -208,7 +207,7 @@ class FreeboxDiskSensor(FreeboxSensor):
         self._unique_id = f"{self._router.mac} {sensor_type} {self._disk['id']} {self._partition['id']}"
 
     @property
-    def device_info(self) -> dict[str, Any]:
+    def device_info(self) -> DeviceInfo:
         """Return the device information."""
         return {
             "identifiers": {(DOMAIN, self._disk["id"])},
