@@ -1,10 +1,11 @@
 """Support for Fritzbox binary sensors."""
-from typing import Callable
+from __future__ import annotations
 
 from homeassistant.components.binary_sensor import (
     DEVICE_CLASS_WINDOW,
     BinarySensorEntity,
 )
+from homeassistant.components.sensor import ATTR_STATE_CLASS
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import (
     ATTR_DEVICE_CLASS,
@@ -13,16 +14,17 @@ from homeassistant.const import (
     ATTR_UNIT_OF_MEASUREMENT,
 )
 from homeassistant.core import HomeAssistant
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
 from . import FritzBoxEntity
 from .const import CONF_COORDINATOR, DOMAIN as FRITZBOX_DOMAIN
 
 
 async def async_setup_entry(
-    hass: HomeAssistant, entry: ConfigEntry, async_add_entities: Callable
+    hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback
 ) -> None:
-    """Set up the Fritzbox binary sensor from ConfigEntry."""
-    entities = []
+    """Set up the FRITZ!SmartHome binary sensor from ConfigEntry."""
+    entities: list[FritzboxBinarySensor] = []
     coordinator = hass.data[FRITZBOX_DOMAIN][entry.entry_id][CONF_COORDINATOR]
 
     for ain, device in coordinator.data.items():
@@ -36,6 +38,7 @@ async def async_setup_entry(
                     ATTR_ENTITY_ID: f"{device.ain}",
                     ATTR_UNIT_OF_MEASUREMENT: None,
                     ATTR_DEVICE_CLASS: DEVICE_CLASS_WINDOW,
+                    ATTR_STATE_CLASS: None,
                 },
                 coordinator,
                 ain,
@@ -46,11 +49,9 @@ async def async_setup_entry(
 
 
 class FritzboxBinarySensor(FritzBoxEntity, BinarySensorEntity):
-    """Representation of a binary Fritzbox device."""
+    """Representation of a binary FRITZ!SmartHome device."""
 
     @property
-    def is_on(self):
+    def is_on(self) -> bool:
         """Return true if sensor is on."""
-        if not self.device.present:
-            return False
-        return self.device.alert_state
+        return self.device.alert_state  # type: ignore [no-any-return]
