@@ -1,4 +1,6 @@
 """Support for Fronius devices."""
+from __future__ import annotations
+
 import copy
 from datetime import timedelta
 import logging
@@ -6,18 +8,26 @@ import logging
 from pyfronius import Fronius
 import voluptuous as vol
 
-from homeassistant.components.sensor import PLATFORM_SCHEMA
+from homeassistant.components.sensor import (
+    PLATFORM_SCHEMA,
+    STATE_CLASS_MEASUREMENT,
+    SensorEntity,
+    SensorEntityDescription,
+)
 from homeassistant.const import (
     CONF_DEVICE,
     CONF_MONITORED_CONDITIONS,
     CONF_RESOURCE,
     CONF_SCAN_INTERVAL,
     CONF_SENSOR_TYPE,
+    DEVICE_CLASS_ENERGY,
+    DEVICE_CLASS_POWER,
 )
+from homeassistant.core import callback
 from homeassistant.helpers.aiohttp_client import async_get_clientsession
 import homeassistant.helpers.config_validation as cv
-from homeassistant.helpers.entity import Entity
 from homeassistant.helpers.event import async_track_time_interval
+from homeassistant.util import dt
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -64,9 +74,7 @@ PLATFORM_SCHEMA = vol.Schema(
                             vol.Optional(CONF_SCOPE, default=DEFAULT_SCOPE): vol.In(
                                 SCOPE_TYPES
                             ),
-                            vol.Optional(CONF_DEVICE): vol.All(
-                                vol.Coerce(int), vol.Range(min=0)
-                            ),
+                            vol.Optional(CONF_DEVICE): cv.positive_int,
                         }
                     ],
                 ),
@@ -90,11 +98,7 @@ async def async_setup_platform(hass, config, async_add_entities, discovery_info=
         device = condition[CONF_DEVICE]
         sensor_type = condition[CONF_SENSOR_TYPE]
         scope = condition[CONF_SCOPE]
-        name = "Fronius {} {} {}".format(
-            condition[CONF_SENSOR_TYPE].replace("_", " ").capitalize(),
-            device if scope == SCOPE_DEVICE else SCOPE_SYSTEM,
-            config[CONF_RESOURCE],
-        )
+        name = f"Fronius {condition[CONF_SENSOR_TYPE].replace('_', ' ').capitalize()} {device if scope == SCOPE_DEVICE else SCOPE_SYSTEM} {config[CONF_RESOURCE]}"
         if sensor_type == TYPE_INVERTER:
             if scope == SCOPE_SYSTEM:
                 adapter_cls = FroniusInverterSystem
@@ -136,6 +140,7 @@ class FroniusAdapter:
         self._name = name
         self._device = device
         self._fetched = {}
+        self._available = True
 
         self.sensors = set()
         self._registered_sensors = set()
@@ -151,21 +156,38 @@ class FroniusAdapter:
         """Return the state attributes."""
         return self._fetched
 
+    @property
+    def available(self):
+        """Whether the fronius device is active."""
+        return self._available
+
+    def entity_description(  # pylint: disable=no-self-use
+        self, key
+    ) -> SensorEntityDescription | None:
+        """Create entity description for a key."""
+        return None
+
     async def async_update(self):
         """Retrieve and update latest state."""
-        values = {}
         try:
             values = await self._update()
         except ConnectionError:
-            _LOGGER.error("Failed to update: connection error")
+            # fronius devices are often powered by self-produced solar energy
+            # and henced turned off at night.
+            # Therefore we will not print multiple errors when connection fails
+            if self._available:
+                self._available = False
+                _LOGGER.error("Failed to update: connection error")
+            return
         except ValueError:
             _LOGGER.error(
                 "Failed to update: invalid response returned."
                 "Maybe the configured device is not supported"
             )
-
-        if not values:
             return
+
+        self._available = True  # reset connection failure
+
         attributes = self._fetched
         # Copy data of current fronius device
         for key, entry in values.items():
@@ -188,18 +210,31 @@ class FroniusAdapter:
         for sensor in self._registered_sensors:
             sensor.async_schedule_update_ha_state(True)
 
-    async def _update(self):
+    async def _update(self) -> dict:
         """Return values of interest."""
-        pass
 
-    async def register(self, sensor):
+    @callback
+    def register(self, sensor):
         """Register child sensor for update subscriptions."""
         self._registered_sensors.add(sensor)
+        return lambda: self._registered_sensors.remove(sensor)
 
 
 class FroniusInverterSystem(FroniusAdapter):
     """Adapter for the fronius inverter with system scope."""
 
+    def entity_description(self, key):
+        """Return the entity descriptor."""
+        if key != "energy_total":
+            return None
+
+        return SensorEntityDescription(
+            key=key,
+            device_class=DEVICE_CLASS_ENERGY,
+            state_class=STATE_CLASS_MEASUREMENT,
+            last_reset=dt.utc_from_timestamp(0),
+        )
+
     async def _update(self):
         """Get the values for the current state."""
         return await self.bridge.current_system_inverter_data()
@@ -208,6 +243,18 @@ class FroniusInverterSystem(FroniusAdapter):
 class FroniusInverterDevice(FroniusAdapter):
     """Adapter for the fronius inverter with device scope."""
 
+    def entity_description(self, key):
+        """Return the entity descriptor."""
+        if key != "energy_total":
+            return None
+
+        return SensorEntityDescription(
+            key=key,
+            device_class=DEVICE_CLASS_ENERGY,
+            state_class=STATE_CLASS_MEASUREMENT,
+            last_reset=dt.utc_from_timestamp(0),
+        )
+
     async def _update(self):
         """Get the values for the current state."""
         return await self.bridge.current_inverter_data(self._device)
@@ -224,6 +271,18 @@ class FroniusStorage(FroniusAdapter):
 class FroniusMeterSystem(FroniusAdapter):
     """Adapter for the fronius meter with system scope."""
 
+    def entity_description(self, key):
+        """Return the entity descriptor."""
+        if not key.startswith("energy_real_"):
+            return None
+
+        return SensorEntityDescription(
+            key=key,
+            device_class=DEVICE_CLASS_ENERGY,
+            state_class=STATE_CLASS_MEASUREMENT,
+            last_reset=dt.utc_from_timestamp(0),
+        )
+
     async def _update(self):
         """Get the values for the current state."""
         return await self.bridge.current_system_meter_data()
@@ -232,6 +291,18 @@ class FroniusMeterSystem(FroniusAdapter):
 class FroniusMeterDevice(FroniusAdapter):
     """Adapter for the fronius meter with device scope."""
 
+    def entity_description(self, key):
+        """Return the entity descriptor."""
+        if not key.startswith("energy_real_"):
+            return None
+
+        return SensorEntityDescription(
+            key=key,
+            device_class=DEVICE_CLASS_ENERGY,
+            state_class=STATE_CLASS_MEASUREMENT,
+            last_reset=dt.utc_from_timestamp(0),
+        )
+
     async def _update(self):
         """Get the values for the current state."""
         return await self.bridge.current_meter_data(self._device)
@@ -240,52 +311,51 @@ class FroniusMeterDevice(FroniusAdapter):
 class FroniusPowerFlow(FroniusAdapter):
     """Adapter for the fronius power flow."""
 
+    def entity_description(self, key):
+        """Return the entity descriptor."""
+        return SensorEntityDescription(
+            key=key,
+            device_class=DEVICE_CLASS_POWER,
+            state_class=STATE_CLASS_MEASUREMENT,
+        )
+
     async def _update(self):
         """Get the values for the current state."""
         return await self.bridge.current_power_flow()
 
 
-class FroniusTemplateSensor(Entity):
+class FroniusTemplateSensor(SensorEntity):
     """Sensor for the single values (e.g. pv power, ac power)."""
 
-    def __init__(self, parent: FroniusAdapter, name):
+    def __init__(self, parent: FroniusAdapter, key):
         """Initialize a singular value sensor."""
-        self._name = name
-        self.parent = parent
-        self._state = None
-        self._unit = None
-
-    @property
-    def name(self):
-        """Return the name of the sensor."""
-        return "{} {}".format(
-            self._name.replace("_", " ").capitalize(), self.parent.name
-        )
-
-    @property
-    def state(self):
-        """Return the current state."""
-        return self._state
-
-    @property
-    def unit_of_measurement(self):
-        """Return the unit of measurement."""
-        return self._unit
+        self._key = key
+        self._attr_name = f"{key.replace('_', ' ').capitalize()} {parent.name}"
+        self._parent = parent
+        if entity_description := parent.entity_description(key):
+            self.entity_description = entity_description
 
     @property
     def should_poll(self):
         """Device should not be polled, returns False."""
         return False
 
+    @property
+    def available(self):
+        """Whether the fronius device is active."""
+        return self._parent.available
+
     async def async_update(self):
         """Update the internal state."""
-        state = self.parent.data.get(self._name)
-        self._state = state.get("value")
-        self._unit = state.get("unit")
+        state = self._parent.data.get(self._key)
+        self._attr_state = state.get("value")
+        if isinstance(self._attr_state, float):
+            self._attr_state = round(self._attr_state, 2)
+        self._attr_unit_of_measurement = state.get("unit")
 
     async def async_added_to_hass(self):
         """Register at parent component for updates."""
-        await self.parent.register(self)
+        self.async_on_remove(self._parent.register(self))
 
     def __hash__(self):
         """Hash sensor by hashing its name."""
