@@ -1,8 +1,8 @@
 """Support for Homekit sensors."""
-from aiohomekit.model.characteristics import CharacteristicsTypes
+from aiohomekit.model.characteristics import Characteristic, CharacteristicsTypes
 from aiohomekit.model.services import ServicesTypes
 
-from homeassistant.components.sensor import SensorEntity
+from homeassistant.components.sensor import STATE_CLASS_MEASUREMENT, SensorEntity
 from homeassistant.const import (
     CONCENTRATION_PARTS_PER_MILLION,
     DEVICE_CLASS_BATTERY,
@@ -12,6 +12,7 @@ from homeassistant.const import (
     DEVICE_CLASS_TEMPERATURE,
     LIGHT_LUX,
     PERCENTAGE,
+    POWER_WATT,
     TEMP_CELSIUS,
 )
 from homeassistant.core import callback
@@ -28,14 +29,40 @@ SIMPLE_SENSOR = {
     CharacteristicsTypes.Vendor.EVE_ENERGY_WATT: {
         "name": "Real Time Energy",
         "device_class": DEVICE_CLASS_POWER,
-        "unit": "watts",
-        "icon": "mdi:chart-line",
+        "state_class": STATE_CLASS_MEASUREMENT,
+        "unit": POWER_WATT,
     },
     CharacteristicsTypes.Vendor.KOOGEEK_REALTIME_ENERGY: {
         "name": "Real Time Energy",
         "device_class": DEVICE_CLASS_POWER,
-        "unit": "watts",
-        "icon": "mdi:chart-line",
+        "state_class": STATE_CLASS_MEASUREMENT,
+        "unit": POWER_WATT,
+    },
+    CharacteristicsTypes.Vendor.KOOGEEK_REALTIME_ENERGY_2: {
+        "name": "Real Time Energy",
+        "device_class": DEVICE_CLASS_POWER,
+        "state_class": STATE_CLASS_MEASUREMENT,
+        "unit": POWER_WATT,
+    },
+    CharacteristicsTypes.get_uuid(CharacteristicsTypes.TEMPERATURE_CURRENT): {
+        "name": "Current Temperature",
+        "device_class": DEVICE_CLASS_TEMPERATURE,
+        "state_class": STATE_CLASS_MEASUREMENT,
+        "unit": TEMP_CELSIUS,
+        # This sensor is only for temperature characteristics that are not part
+        # of a temperature sensor service.
+        "probe": lambda char: char.service.type
+        != ServicesTypes.get_uuid(ServicesTypes.TEMPERATURE_SENSOR),
+    },
+    CharacteristicsTypes.get_uuid(CharacteristicsTypes.RELATIVE_HUMIDITY_CURRENT): {
+        "name": "Current Humidity",
+        "device_class": DEVICE_CLASS_HUMIDITY,
+        "state_class": STATE_CLASS_MEASUREMENT,
+        "unit": PERCENTAGE,
+        # This sensor is only for humidity characteristics that are not part
+        # of a humidity sensor service.
+        "probe": lambda char: char.service.type
+        != ServicesTypes.get_uuid(ServicesTypes.HUMIDITY_SENSOR),
     },
 }
 
@@ -43,15 +70,13 @@ SIMPLE_SENSOR = {
 class HomeKitHumiditySensor(HomeKitEntity, SensorEntity):
     """Representation of a Homekit humidity sensor."""
 
+    _attr_device_class = DEVICE_CLASS_HUMIDITY
+    _attr_unit_of_measurement = PERCENTAGE
+
     def get_characteristic_types(self):
         """Define the homekit characteristics the entity is tracking."""
         return [CharacteristicsTypes.RELATIVE_HUMIDITY_CURRENT]
 
-    @property
-    def device_class(self) -> str:
-        """Return the device class of the sensor."""
-        return DEVICE_CLASS_HUMIDITY
-
     @property
     def name(self):
         """Return the name of the device."""
@@ -62,11 +87,6 @@ class HomeKitHumiditySensor(HomeKitEntity, SensorEntity):
         """Return the sensor icon."""
         return HUMIDITY_ICON
 
-    @property
-    def unit_of_measurement(self):
-        """Return units for the sensor."""
-        return PERCENTAGE
-
     @property
     def state(self):
         """Return the current humidity."""
@@ -76,15 +96,13 @@ class HomeKitHumiditySensor(HomeKitEntity, SensorEntity):
 class HomeKitTemperatureSensor(HomeKitEntity, SensorEntity):
     """Representation of a Homekit temperature sensor."""
 
+    _attr_device_class = DEVICE_CLASS_TEMPERATURE
+    _attr_unit_of_measurement = TEMP_CELSIUS
+
     def get_characteristic_types(self):
         """Define the homekit characteristics the entity is tracking."""
         return [CharacteristicsTypes.TEMPERATURE_CURRENT]
 
-    @property
-    def device_class(self) -> str:
-        """Return the device class of the sensor."""
-        return DEVICE_CLASS_TEMPERATURE
-
     @property
     def name(self):
         """Return the name of the device."""
@@ -95,11 +113,6 @@ class HomeKitTemperatureSensor(HomeKitEntity, SensorEntity):
         """Return the sensor icon."""
         return TEMP_C_ICON
 
-    @property
-    def unit_of_measurement(self):
-        """Return units for the sensor."""
-        return TEMP_CELSIUS
-
     @property
     def state(self):
         """Return the current temperature in Celsius."""
@@ -109,15 +122,13 @@ class HomeKitTemperatureSensor(HomeKitEntity, SensorEntity):
 class HomeKitLightSensor(HomeKitEntity, SensorEntity):
     """Representation of a Homekit light level sensor."""
 
+    _attr_device_class = DEVICE_CLASS_ILLUMINANCE
+    _attr_unit_of_measurement = LIGHT_LUX
+
     def get_characteristic_types(self):
         """Define the homekit characteristics the entity is tracking."""
         return [CharacteristicsTypes.LIGHT_LEVEL_CURRENT]
 
-    @property
-    def device_class(self) -> str:
-        """Return the device class of the sensor."""
-        return DEVICE_CLASS_ILLUMINANCE
-
     @property
     def name(self):
         """Return the name of the device."""
@@ -128,11 +139,6 @@ class HomeKitLightSensor(HomeKitEntity, SensorEntity):
         """Return the sensor icon."""
         return BRIGHTNESS_ICON
 
-    @property
-    def unit_of_measurement(self):
-        """Return units for the sensor."""
-        return LIGHT_LUX
-
     @property
     def state(self):
         """Return the current light level in lux."""
@@ -142,6 +148,9 @@ class HomeKitLightSensor(HomeKitEntity, SensorEntity):
 class HomeKitCarbonDioxideSensor(HomeKitEntity, SensorEntity):
     """Representation of a Homekit Carbon Dioxide sensor."""
 
+    _attr_icon = CO2_ICON
+    _attr_unit_of_measurement = CONCENTRATION_PARTS_PER_MILLION
+
     def get_characteristic_types(self):
         """Define the homekit characteristics the entity is tracking."""
         return [CharacteristicsTypes.CARBON_DIOXIDE_LEVEL]
@@ -151,16 +160,6 @@ class HomeKitCarbonDioxideSensor(HomeKitEntity, SensorEntity):
         """Return the name of the device."""
         return f"{super().name} CO2"
 
-    @property
-    def icon(self):
-        """Return the sensor icon."""
-        return CO2_ICON
-
-    @property
-    def unit_of_measurement(self):
-        """Return units for the sensor."""
-        return CONCENTRATION_PARTS_PER_MILLION
-
     @property
     def state(self):
         """Return the current CO2 level in ppm."""
@@ -170,6 +169,9 @@ class HomeKitCarbonDioxideSensor(HomeKitEntity, SensorEntity):
 class HomeKitBatterySensor(HomeKitEntity, SensorEntity):
     """Representation of a Homekit battery sensor."""
 
+    _attr_device_class = DEVICE_CLASS_BATTERY
+    _attr_unit_of_measurement = PERCENTAGE
+
     def get_characteristic_types(self):
         """Define the homekit characteristics the entity is tracking."""
         return [
@@ -178,11 +180,6 @@ class HomeKitBatterySensor(HomeKitEntity, SensorEntity):
             CharacteristicsTypes.CHARGING_STATE,
         ]
 
-    @property
-    def device_class(self) -> str:
-        """Return the device class of the sensor."""
-        return DEVICE_CLASS_BATTERY
-
     @property
     def name(self):
         """Return the name of the device."""
@@ -210,11 +207,6 @@ class HomeKitBatterySensor(HomeKitEntity, SensorEntity):
 
         return icon
 
-    @property
-    def unit_of_measurement(self):
-        """Return units for the sensor."""
-        return PERCENTAGE
-
     @property
     def is_low_battery(self):
         """Return true if battery level is low."""
@@ -251,18 +243,20 @@ class SimpleSensor(CharacteristicEntity, SensorEntity):
         info,
         char,
         device_class=None,
+        state_class=None,
         unit=None,
         icon=None,
         name=None,
+        **kwargs,
     ):
         """Initialise a secondary HomeKit characteristic sensor."""
         self._device_class = device_class
+        self._state_class = state_class
         self._unit = unit
         self._icon = icon
         self._name = name
-        self._char = char
 
-        super().__init__(conn, info)
+        super().__init__(conn, info, char)
 
     def get_characteristic_types(self):
         """Define the homekit characteristics the entity is tracking."""
@@ -270,9 +264,14 @@ class SimpleSensor(CharacteristicEntity, SensorEntity):
 
     @property
     def device_class(self):
-        """Return units for the sensor."""
+        """Return type of sensor."""
         return self._device_class
 
+    @property
+    def state_class(self):
+        """Return type of state."""
+        return self._state_class
+
     @property
     def unit_of_measurement(self):
         """Return units for the sensor."""
@@ -320,10 +319,12 @@ async def async_setup_entry(hass, config_entry, async_add_entities):
     conn.add_listener(async_add_service)
 
     @callback
-    def async_add_characteristic(char):
+    def async_add_characteristic(char: Characteristic):
         kwargs = SIMPLE_SENSOR.get(char.type)
         if not kwargs:
             return False
+        if "probe" in kwargs and not kwargs["probe"](char):
+            return False
         info = {"aid": char.service.accessory.aid, "iid": char.service.iid}
         async_add_entities([SimpleSensor(conn, info, char, **kwargs)], True)
 
