@@ -8,6 +8,8 @@ from homeassistant.const import (
     DEVICE_CLASS_ILLUMINANCE,
     DEVICE_CLASS_POWER,
     DEVICE_CLASS_TEMPERATURE,
+    ELECTRIC_CURRENT_MILLIAMPERE,
+    ELECTRIC_POTENTIAL_VOLT,
     ENERGY_WATT_HOUR,
     FREQUENCY_HERTZ,
     LENGTH_MILLIMETERS,
@@ -17,7 +19,6 @@ from homeassistant.const import (
     PRESSURE_HPA,
     SPEED_KILOMETERS_PER_HOUR,
     TEMP_CELSIUS,
-    VOLT,
     VOLUME_CUBIC_METERS,
 )
 
@@ -47,11 +48,13 @@ HM_UNIT_HA_CAST = {
     "ACTUAL_TEMPERATURE": TEMP_CELSIUS,
     "BRIGHTNESS": "#",
     "POWER": POWER_WATT,
-    "CURRENT": "mA",
-    "VOLTAGE": VOLT,
+    "CURRENT": ELECTRIC_CURRENT_MILLIAMPERE,
+    "VOLTAGE": ELECTRIC_POTENTIAL_VOLT,
     "ENERGY_COUNTER": ENERGY_WATT_HOUR,
     "GAS_POWER": VOLUME_CUBIC_METERS,
     "GAS_ENERGY_COUNTER": VOLUME_CUBIC_METERS,
+    "IEC_POWER": POWER_WATT,
+    "IEC_ENERGY_COUNTER": ENERGY_WATT_HOUR,
     "LUX": LIGHT_LUX,
     "ILLUMINATION": LIGHT_LUX,
     "CURRENT_ILLUMINATION": LIGHT_LUX,
