@@ -2,7 +2,6 @@
 from __future__ import annotations
 
 import logging
-from typing import Any
 
 import voluptuous as vol
 
@@ -55,12 +54,12 @@ from .const import (
     CALL_TYPE_DISCRETE,
     CALL_TYPE_REGISTER_HOLDING,
     CALL_TYPE_REGISTER_INPUT,
+    CALL_TYPE_X_COILS,
+    CALL_TYPE_X_REGISTER_HOLDINGS,
     CONF_BAUDRATE,
     CONF_BYTESIZE,
     CONF_CLIMATES,
     CONF_CLOSE_COMM_ON_ERROR,
-    CONF_CURRENT_TEMP,
-    CONF_CURRENT_TEMP_REGISTER_TYPE,
     CONF_DATA_COUNT,
     CONF_DATA_TYPE,
     CONF_FANS,
@@ -69,9 +68,12 @@ from .const import (
     CONF_MIN_TEMP,
     CONF_PARITY,
     CONF_PRECISION,
-    CONF_REGISTER,
+    CONF_RETRIES,
+    CONF_RETRY_ON_EMPTY,
     CONF_REVERSE_ORDER,
+    CONF_RTUOVERTCP,
     CONF_SCALE,
+    CONF_SERIAL,
     CONF_STATE_CLOSED,
     CONF_STATE_CLOSING,
     CONF_STATE_OFF,
@@ -88,86 +90,41 @@ from .const import (
     CONF_SWAP_WORD,
     CONF_SWAP_WORD_BYTE,
     CONF_TARGET_TEMP,
+    CONF_TCP,
+    CONF_UDP,
     CONF_VERIFY,
     CONF_WRITE_TYPE,
     DATA_TYPE_CUSTOM,
     DATA_TYPE_FLOAT,
+    DATA_TYPE_FLOAT16,
+    DATA_TYPE_FLOAT32,
+    DATA_TYPE_FLOAT64,
     DATA_TYPE_INT,
+    DATA_TYPE_INT16,
+    DATA_TYPE_INT32,
+    DATA_TYPE_INT64,
     DATA_TYPE_STRING,
     DATA_TYPE_UINT,
+    DATA_TYPE_UINT16,
+    DATA_TYPE_UINT32,
+    DATA_TYPE_UINT64,
     DEFAULT_HUB,
     DEFAULT_SCAN_INTERVAL,
-    DEFAULT_STRUCTURE_PREFIX,
     DEFAULT_TEMP_UNIT,
     MODBUS_DOMAIN as DOMAIN,
-    PLATFORMS,
 )
 from .modbus import async_modbus_setup
+from .validators import number_validator, scan_interval_validator, struct_validator
 
 _LOGGER = logging.getLogger(__name__)
 
 BASE_SCHEMA = vol.Schema({vol.Optional(CONF_NAME, default=DEFAULT_HUB): cv.string})
 
 
-def number(value: Any) -> int | float:
-    """Coerce a value to number without losing precision."""
-    if isinstance(value, int):
-        return value
-    if isinstance(value, float):
-        return value
-
-    try:
-        value = int(value)
-        return value
-    except (TypeError, ValueError):
-        pass
-    try:
-        value = float(value)
-        return value
-    except (TypeError, ValueError) as err:
-        raise vol.Invalid(f"invalid number {value}") from err
-
-
-def control_scan_interval(config: dict) -> dict:
-    """Control scan_interval."""
-    for hub in config:
-        minimum_scan_interval = DEFAULT_SCAN_INTERVAL
-        for component, conf_key in PLATFORMS:
-            if conf_key not in hub:
-                continue
-
-            for entry in hub[conf_key]:
-                scan_interval = entry.get(CONF_SCAN_INTERVAL, DEFAULT_SCAN_INTERVAL)
-                if scan_interval == 0:
-                    continue
-                if scan_interval < 5:
-                    _LOGGER.warning(
-                        "%s %s scan_interval(%d) is lower than 5 seconds, "
-                        "which may cause Home Assistant stability issues",
-                        component,
-                        entry.get(CONF_NAME),
-                        scan_interval,
-                    )
-                entry[CONF_SCAN_INTERVAL] = scan_interval
-                minimum_scan_interval = min(scan_interval, minimum_scan_interval)
-        if (
-            CONF_TIMEOUT in hub
-            and hub[CONF_TIMEOUT] > minimum_scan_interval - 1
-            and minimum_scan_interval > 1
-        ):
-            _LOGGER.warning(
-                "Modbus %s timeout(%d) is adjusted(%d) due to scan_interval",
-                hub.get(CONF_NAME, ""),
-                hub[CONF_TIMEOUT],
-                minimum_scan_interval - 1,
-            )
-            hub[CONF_TIMEOUT] = minimum_scan_interval - 1
-    return config
-
-
 BASE_COMPONENT_SCHEMA = vol.Schema(
     {
         vol.Required(CONF_NAME): cv.string,
+        vol.Required(CONF_ADDRESS): cv.positive_int,
         vol.Optional(CONF_SLAVE): cv.positive_int,
         vol.Optional(
             CONF_SCAN_INTERVAL, default=DEFAULT_SCAN_INTERVAL
@@ -176,54 +133,59 @@ BASE_COMPONENT_SCHEMA = vol.Schema(
 )
 
 
-CLIMATE_SCHEMA = BASE_COMPONENT_SCHEMA.extend(
+BASE_STRUCT_SCHEMA = BASE_COMPONENT_SCHEMA.extend(
     {
-        vol.Required(CONF_CURRENT_TEMP): cv.positive_int,
-        vol.Required(CONF_TARGET_TEMP): cv.positive_int,
-        vol.Optional(CONF_DATA_COUNT, default=2): cv.positive_int,
-        vol.Optional(
-            CONF_CURRENT_TEMP_REGISTER_TYPE, default=CALL_TYPE_REGISTER_HOLDING
-        ): vol.In([CALL_TYPE_REGISTER_HOLDING, CALL_TYPE_REGISTER_INPUT]),
-        vol.Optional(CONF_DATA_TYPE, default=DATA_TYPE_FLOAT): vol.In(
-            [DATA_TYPE_INT, DATA_TYPE_UINT, DATA_TYPE_FLOAT, DATA_TYPE_CUSTOM]
+        vol.Optional(CONF_INPUT_TYPE, default=CALL_TYPE_REGISTER_HOLDING): vol.In(
+            [
+                CALL_TYPE_REGISTER_HOLDING,
+                CALL_TYPE_REGISTER_INPUT,
+            ]
+        ),
+        vol.Optional(CONF_COUNT): cv.positive_int,
+        vol.Optional(CONF_DATA_TYPE, default=DATA_TYPE_INT): vol.In(
+            [
+                DATA_TYPE_INT16,
+                DATA_TYPE_INT32,
+                DATA_TYPE_INT64,
+                DATA_TYPE_UINT16,
+                DATA_TYPE_UINT32,
+                DATA_TYPE_UINT64,
+                DATA_TYPE_FLOAT16,
+                DATA_TYPE_FLOAT32,
+                DATA_TYPE_FLOAT64,
+                DATA_TYPE_STRING,
+                DATA_TYPE_INT,
+                DATA_TYPE_UINT,
+                DATA_TYPE_FLOAT,
+                DATA_TYPE_STRING,
+                DATA_TYPE_CUSTOM,
+            ]
+        ),
+        vol.Optional(CONF_STRUCTURE): cv.string,
+        vol.Optional(CONF_SCALE, default=1): number_validator,
+        vol.Optional(CONF_OFFSET, default=0): number_validator,
+        vol.Optional(CONF_PRECISION, default=0): cv.positive_int,
+        vol.Optional(CONF_SWAP, default=CONF_SWAP_NONE): vol.In(
+            [
+                CONF_SWAP_NONE,
+                CONF_SWAP_BYTE,
+                CONF_SWAP_WORD,
+                CONF_SWAP_WORD_BYTE,
+            ]
         ),
-        vol.Optional(CONF_PRECISION, default=1): cv.positive_int,
-        vol.Optional(CONF_SCALE, default=1): vol.Coerce(float),
-        vol.Optional(CONF_OFFSET, default=0): vol.Coerce(float),
-        vol.Optional(CONF_MAX_TEMP, default=35): cv.positive_int,
-        vol.Optional(CONF_MIN_TEMP, default=5): cv.positive_int,
-        vol.Optional(CONF_STEP, default=0.5): vol.Coerce(float),
-        vol.Optional(CONF_STRUCTURE, default=DEFAULT_STRUCTURE_PREFIX): cv.string,
-        vol.Optional(CONF_TEMPERATURE_UNIT, default=DEFAULT_TEMP_UNIT): cv.string,
     }
 )
 
-COVERS_SCHEMA = vol.All(
-    cv.has_at_least_one_key(CALL_TYPE_COIL, CONF_REGISTER),
-    BASE_COMPONENT_SCHEMA.extend(
-        {
-            vol.Optional(CONF_DEVICE_CLASS): COVER_DEVICE_CLASSES_SCHEMA,
-            vol.Optional(CONF_STATE_CLOSED, default=0): cv.positive_int,
-            vol.Optional(CONF_STATE_CLOSING, default=3): cv.positive_int,
-            vol.Optional(CONF_STATE_OPEN, default=1): cv.positive_int,
-            vol.Optional(CONF_STATE_OPENING, default=2): cv.positive_int,
-            vol.Optional(CONF_STATUS_REGISTER): cv.positive_int,
-            vol.Optional(
-                CONF_STATUS_REGISTER_TYPE,
-                default=CALL_TYPE_REGISTER_HOLDING,
-            ): vol.In([CALL_TYPE_REGISTER_HOLDING, CALL_TYPE_REGISTER_INPUT]),
-            vol.Exclusive(CALL_TYPE_COIL, CONF_INPUT_TYPE): cv.positive_int,
-            vol.Exclusive(CONF_REGISTER, CONF_INPUT_TYPE): cv.positive_int,
-        }
-    ),
-)
 
-SWITCH_SCHEMA = BASE_COMPONENT_SCHEMA.extend(
+BASE_SWITCH_SCHEMA = BASE_COMPONENT_SCHEMA.extend(
     {
-        vol.Required(CONF_ADDRESS): cv.positive_int,
-        vol.Optional(CONF_DEVICE_CLASS): SWITCH_DEVICE_CLASSES_SCHEMA,
         vol.Optional(CONF_WRITE_TYPE, default=CALL_TYPE_REGISTER_HOLDING): vol.In(
-            [CALL_TYPE_REGISTER_HOLDING, CALL_TYPE_COIL]
+            [
+                CALL_TYPE_REGISTER_HOLDING,
+                CALL_TYPE_COIL,
+                CALL_TYPE_X_COILS,
+                CALL_TYPE_X_REGISTER_HOLDINGS,
+            ]
         ),
         vol.Optional(CONF_COMMAND_OFF, default=0x00): cv.positive_int,
         vol.Optional(CONF_COMMAND_ON, default=0x01): cv.positive_int,
@@ -236,6 +198,8 @@ SWITCH_SCHEMA = BASE_COMPONENT_SCHEMA.extend(
                         CALL_TYPE_DISCRETE,
                         CALL_TYPE_REGISTER_INPUT,
                         CALL_TYPE_COIL,
+                        CALL_TYPE_X_COILS,
+                        CALL_TYPE_X_REGISTER_HOLDINGS,
                     ]
                 ),
                 vol.Optional(CONF_STATE_OFF): cv.positive_int,
@@ -246,90 +210,64 @@ SWITCH_SCHEMA = BASE_COMPONENT_SCHEMA.extend(
     }
 )
 
-LIGHT_SCHEMA = BASE_COMPONENT_SCHEMA.extend(
-    {
-        vol.Required(CONF_ADDRESS): cv.positive_int,
-        vol.Optional(CONF_WRITE_TYPE, default=CALL_TYPE_REGISTER_HOLDING): vol.In(
-            [CALL_TYPE_REGISTER_HOLDING, CALL_TYPE_COIL]
-        ),
-        vol.Optional(CONF_COMMAND_OFF, default=0x00): cv.positive_int,
-        vol.Optional(CONF_COMMAND_ON, default=0x01): cv.positive_int,
-        vol.Optional(CONF_VERIFY): vol.Maybe(
-            {
-                vol.Optional(CONF_ADDRESS): cv.positive_int,
-                vol.Optional(CONF_INPUT_TYPE): vol.In(
-                    [
-                        CALL_TYPE_REGISTER_HOLDING,
-                        CALL_TYPE_DISCRETE,
-                        CALL_TYPE_REGISTER_INPUT,
-                        CALL_TYPE_COIL,
-                    ]
-                ),
-                vol.Optional(CONF_STATE_OFF): cv.positive_int,
-                vol.Optional(CONF_STATE_ON): cv.positive_int,
-            }
-        ),
-    }
+
+CLIMATE_SCHEMA = vol.All(
+    cv.deprecated(CONF_DATA_COUNT, replacement_key=CONF_COUNT),
+    BASE_STRUCT_SCHEMA.extend(
+        {
+            vol.Required(CONF_TARGET_TEMP): cv.positive_int,
+            vol.Optional(CONF_MAX_TEMP, default=35): cv.positive_int,
+            vol.Optional(CONF_MIN_TEMP, default=5): cv.positive_int,
+            vol.Optional(CONF_STEP, default=0.5): vol.Coerce(float),
+            vol.Optional(CONF_TEMPERATURE_UNIT, default=DEFAULT_TEMP_UNIT): cv.string,
+        }
+    ),
 )
 
-FAN_SCHEMA = BASE_COMPONENT_SCHEMA.extend(
+COVERS_SCHEMA = BASE_COMPONENT_SCHEMA.extend(
     {
-        vol.Required(CONF_ADDRESS): cv.positive_int,
-        vol.Optional(CONF_WRITE_TYPE, default=CALL_TYPE_REGISTER_HOLDING): vol.In(
-            [CALL_TYPE_REGISTER_HOLDING, CALL_TYPE_COIL]
-        ),
-        vol.Optional(CONF_COMMAND_OFF, default=0x00): cv.positive_int,
-        vol.Optional(CONF_COMMAND_ON, default=0x01): cv.positive_int,
-        vol.Optional(CONF_VERIFY): vol.Maybe(
-            {
-                vol.Optional(CONF_ADDRESS): cv.positive_int,
-                vol.Optional(CONF_INPUT_TYPE): vol.In(
-                    [
-                        CALL_TYPE_REGISTER_HOLDING,
-                        CALL_TYPE_DISCRETE,
-                        CALL_TYPE_REGISTER_INPUT,
-                        CALL_TYPE_COIL,
-                    ]
-                ),
-                vol.Optional(CONF_STATE_OFF): cv.positive_int,
-                vol.Optional(CONF_STATE_ON): cv.positive_int,
-            }
-        ),
-    }
-)
-
-SENSOR_SCHEMA = BASE_COMPONENT_SCHEMA.extend(
-    {
-        vol.Required(CONF_ADDRESS): cv.positive_int,
-        vol.Optional(CONF_COUNT, default=1): cv.positive_int,
-        vol.Optional(CONF_DATA_TYPE, default=DATA_TYPE_INT): vol.In(
+        vol.Optional(CONF_INPUT_TYPE, default=CALL_TYPE_REGISTER_HOLDING,): vol.In(
             [
-                DATA_TYPE_INT,
-                DATA_TYPE_UINT,
-                DATA_TYPE_FLOAT,
-                DATA_TYPE_STRING,
-                DATA_TYPE_CUSTOM,
+                CALL_TYPE_REGISTER_HOLDING,
+                CALL_TYPE_COIL,
             ]
         ),
-        vol.Optional(CONF_DEVICE_CLASS): SENSOR_DEVICE_CLASSES_SCHEMA,
-        vol.Optional(CONF_OFFSET, default=0): number,
-        vol.Optional(CONF_PRECISION, default=0): cv.positive_int,
-        vol.Optional(CONF_INPUT_TYPE, default=CALL_TYPE_REGISTER_HOLDING): vol.In(
-            [CALL_TYPE_REGISTER_HOLDING, CALL_TYPE_REGISTER_INPUT]
-        ),
-        vol.Optional(CONF_REVERSE_ORDER): cv.boolean,
-        vol.Optional(CONF_SWAP, default=CONF_SWAP_NONE): vol.In(
-            [CONF_SWAP_NONE, CONF_SWAP_BYTE, CONF_SWAP_WORD, CONF_SWAP_WORD_BYTE]
-        ),
-        vol.Optional(CONF_SCALE, default=1): number,
-        vol.Optional(CONF_STRUCTURE): cv.string,
-        vol.Optional(CONF_UNIT_OF_MEASUREMENT): cv.string,
+        vol.Optional(CONF_DEVICE_CLASS): COVER_DEVICE_CLASSES_SCHEMA,
+        vol.Optional(CONF_STATE_CLOSED, default=0): cv.positive_int,
+        vol.Optional(CONF_STATE_CLOSING, default=3): cv.positive_int,
+        vol.Optional(CONF_STATE_OPEN, default=1): cv.positive_int,
+        vol.Optional(CONF_STATE_OPENING, default=2): cv.positive_int,
+        vol.Optional(CONF_STATUS_REGISTER): cv.positive_int,
+        vol.Optional(
+            CONF_STATUS_REGISTER_TYPE,
+            default=CALL_TYPE_REGISTER_HOLDING,
+        ): vol.In([CALL_TYPE_REGISTER_HOLDING, CALL_TYPE_REGISTER_INPUT]),
     }
 )
 
+SWITCH_SCHEMA = BASE_SWITCH_SCHEMA.extend(
+    {
+        vol.Optional(CONF_DEVICE_CLASS): SWITCH_DEVICE_CLASSES_SCHEMA,
+    }
+)
+
+LIGHT_SCHEMA = BASE_SWITCH_SCHEMA.extend({})
+
+FAN_SCHEMA = BASE_SWITCH_SCHEMA.extend({})
+
+SENSOR_SCHEMA = vol.All(
+    cv.deprecated(CONF_REVERSE_ORDER),
+    BASE_STRUCT_SCHEMA.extend(
+        {
+            vol.Optional(CONF_DEVICE_CLASS): SENSOR_DEVICE_CLASSES_SCHEMA,
+            vol.Optional(CONF_UNIT_OF_MEASUREMENT): cv.string,
+            vol.Optional(CONF_REVERSE_ORDER): cv.boolean,
+        }
+    ),
+)
+
 BINARY_SENSOR_SCHEMA = BASE_COMPONENT_SCHEMA.extend(
     {
-        vol.Required(CONF_ADDRESS): cv.positive_int,
         vol.Optional(CONF_DEVICE_CLASS): BINARY_SENSOR_DEVICE_CLASSES_SCHEMA,
         vol.Optional(CONF_INPUT_TYPE, default=CALL_TYPE_COIL): vol.In(
             [CALL_TYPE_COIL, CALL_TYPE_DISCRETE]
@@ -343,13 +281,19 @@ MODBUS_SCHEMA = vol.Schema(
         vol.Optional(CONF_TIMEOUT, default=3): cv.socket_timeout,
         vol.Optional(CONF_CLOSE_COMM_ON_ERROR, default=True): cv.boolean,
         vol.Optional(CONF_DELAY, default=0): cv.positive_int,
+        vol.Optional(CONF_RETRIES, default=3): cv.positive_int,
+        vol.Optional(CONF_RETRY_ON_EMPTY, default=False): cv.boolean,
         vol.Optional(CONF_BINARY_SENSORS): vol.All(
             cv.ensure_list, [BINARY_SENSOR_SCHEMA]
         ),
-        vol.Optional(CONF_CLIMATES): vol.All(cv.ensure_list, [CLIMATE_SCHEMA]),
+        vol.Optional(CONF_CLIMATES): vol.All(
+            cv.ensure_list, [vol.All(CLIMATE_SCHEMA, struct_validator)]
+        ),
         vol.Optional(CONF_COVERS): vol.All(cv.ensure_list, [COVERS_SCHEMA]),
         vol.Optional(CONF_LIGHTS): vol.All(cv.ensure_list, [LIGHT_SCHEMA]),
-        vol.Optional(CONF_SENSORS): vol.All(cv.ensure_list, [SENSOR_SCHEMA]),
+        vol.Optional(CONF_SENSORS): vol.All(
+            cv.ensure_list, [vol.All(SENSOR_SCHEMA, struct_validator)]
+        ),
         vol.Optional(CONF_SWITCHES): vol.All(cv.ensure_list, [SWITCH_SCHEMA]),
         vol.Optional(CONF_FANS): vol.All(cv.ensure_list, [FAN_SCHEMA]),
     }
@@ -357,7 +301,7 @@ MODBUS_SCHEMA = vol.Schema(
 
 SERIAL_SCHEMA = MODBUS_SCHEMA.extend(
     {
-        vol.Required(CONF_TYPE): "serial",
+        vol.Required(CONF_TYPE): CONF_SERIAL,
         vol.Required(CONF_BAUDRATE): cv.positive_int,
         vol.Required(CONF_BYTESIZE): vol.Any(5, 6, 7, 8),
         vol.Required(CONF_METHOD): vol.Any("rtu", "ascii"),
@@ -371,7 +315,7 @@ ETHERNET_SCHEMA = MODBUS_SCHEMA.extend(
     {
         vol.Required(CONF_HOST): cv.string,
         vol.Required(CONF_PORT): cv.port,
-        vol.Required(CONF_TYPE): vol.Any("tcp", "udp", "rtuovertcp"),
+        vol.Required(CONF_TYPE): vol.Any(CONF_TCP, CONF_UDP, CONF_RTUOVERTCP),
     }
 )
 
@@ -379,7 +323,7 @@ CONFIG_SCHEMA = vol.Schema(
     {
         DOMAIN: vol.All(
             cv.ensure_list,
-            control_scan_interval,
+            scan_interval_validator,
             [
                 vol.Any(SERIAL_SCHEMA, ETHERNET_SCHEMA),
             ],
