@@ -1,64 +1,36 @@
 """Support for Nexia / Trane XL Thermostats."""
-import asyncio
 from datetime import timedelta
 from functools import partial
 import logging
 
+from nexia.const import BRAND_NEXIA
 from nexia.home import NexiaHome
 from requests.exceptions import ConnectTimeout, HTTPError
-import voluptuous as vol
 
-from homeassistant.config_entries import SOURCE_IMPORT, ConfigEntry
+from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import CONF_PASSWORD, CONF_USERNAME
 from homeassistant.core import HomeAssistant
 from homeassistant.exceptions import ConfigEntryNotReady
 import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers.update_coordinator import DataUpdateCoordinator
 
-from .const import DOMAIN, NEXIA_DEVICE, PLATFORMS, UPDATE_COORDINATOR
+from .const import CONF_BRAND, DOMAIN, NEXIA_DEVICE, PLATFORMS, UPDATE_COORDINATOR
+from .util import is_invalid_auth_code
 
 _LOGGER = logging.getLogger(__name__)
 
-
-CONFIG_SCHEMA = vol.Schema(
-    {
-        DOMAIN: vol.Schema(
-            {
-                vol.Required(CONF_USERNAME): cv.string,
-                vol.Required(CONF_PASSWORD): cv.string,
-            },
-            extra=vol.ALLOW_EXTRA,
-        ),
-    },
-    extra=vol.ALLOW_EXTRA,
-)
+CONFIG_SCHEMA = cv.deprecated(DOMAIN)
 
 DEFAULT_UPDATE_RATE = 120
 
 
-async def async_setup(hass: HomeAssistant, config: dict) -> bool:
-    """Set up the nexia component from YAML."""
-
-    conf = config.get(DOMAIN)
-    hass.data.setdefault(DOMAIN, {})
-
-    if not conf:
-        return True
-
-    hass.async_create_task(
-        hass.config_entries.flow.async_init(
-            DOMAIN, context={"source": SOURCE_IMPORT}, data=conf
-        )
-    )
-    return True
-
-
-async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
+async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Configure the base Nexia device for Home Assistant."""
 
     conf = entry.data
     username = conf[CONF_USERNAME]
     password = conf[CONF_PASSWORD]
+    brand = conf.get(CONF_BRAND, BRAND_NEXIA)
 
     state_file = hass.config.path(f"nexia_config_{username}.conf")
 
@@ -70,24 +42,24 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
                 password=password,
                 device_name=hass.config.location_name,
                 state_file=state_file,
+                brand=brand,
             )
         )
     except ConnectTimeout as ex:
         _LOGGER.error("Unable to connect to Nexia service: %s", ex)
-        raise ConfigEntryNotReady
+        raise ConfigEntryNotReady from ex
     except HTTPError as http_ex:
-        if http_ex.response.status_code >= 400 and http_ex.response.status_code < 500:
+        if is_invalid_auth_code(http_ex.response.status_code):
             _LOGGER.error(
-                "Access error from Nexia service, please check credentials: %s",
-                http_ex,
+                "Access error from Nexia service, please check credentials: %s", http_ex
             )
             return False
         _LOGGER.error("HTTP error from Nexia service: %s", http_ex)
-        raise ConfigEntryNotReady
+        raise ConfigEntryNotReady from http_ex
 
     async def _async_update_data():
         """Fetch data from API endpoint."""
-        return await hass.async_add_job(nexia_home.update)
+        return await hass.async_add_executor_job(nexia_home.update)
 
     coordinator = DataUpdateCoordinator(
         hass,
@@ -97,29 +69,20 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
         update_interval=timedelta(seconds=DEFAULT_UPDATE_RATE),
     )
 
+    hass.data.setdefault(DOMAIN, {})
     hass.data[DOMAIN][entry.entry_id] = {
         NEXIA_DEVICE: nexia_home,
         UPDATE_COORDINATOR: coordinator,
     }
 
-    for component in PLATFORMS:
-        hass.async_create_task(
-            hass.config_entries.async_forward_entry_setup(entry, component)
-        )
+    hass.config_entries.async_setup_platforms(entry, PLATFORMS)
 
     return True
 
 
 async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry):
     """Unload a config entry."""
-    unload_ok = all(
-        await asyncio.gather(
-            *[
-                hass.config_entries.async_forward_entry_unload(entry, component)
-                for component in PLATFORMS
-            ]
-        )
-    )
+    unload_ok = await hass.config_entries.async_unload_platforms(entry, PLATFORMS)
     if unload_ok:
         hass.data[DOMAIN].pop(entry.entry_id)
 
