@@ -11,13 +11,13 @@ from typing import Any, Callable
 import urllib.parse
 
 import async_timeout
-from pysonos.core import MUSIC_SRC_LINE_IN, MUSIC_SRC_RADIO, MUSIC_SRC_TV, SoCo
-from pysonos.data_structures import DidlAudioBroadcast, DidlPlaylistContainer
-from pysonos.events_base import Event as SonosEvent, SubscriptionBase
-from pysonos.exceptions import SoCoException
-from pysonos.music_library import MusicLibrary
-from pysonos.plugins.sharelink import ShareLinkPlugin
-from pysonos.snapshot import Snapshot
+from soco.core import MUSIC_SRC_LINE_IN, MUSIC_SRC_RADIO, MUSIC_SRC_TV, SoCo
+from soco.data_structures import DidlAudioBroadcast, DidlPlaylistContainer
+from soco.events_base import Event as SonosEvent, SubscriptionBase
+from soco.exceptions import SoCoException, SoCoUPnPException
+from soco.music_library import MusicLibrary
+from soco.plugins.sharelink import ShareLinkPlugin
+from soco.snapshot import Snapshot
 
 from homeassistant.components import zeroconf
 from homeassistant.components.binary_sensor import DOMAIN as BINARY_SENSOR_DOMAIN
@@ -25,6 +25,7 @@ from homeassistant.components.media_player import DOMAIN as MP_DOMAIN
 from homeassistant.components.sensor import DOMAIN as SENSOR_DOMAIN
 from homeassistant.components.switch import DOMAIN as SWITCH_DOMAIN
 from homeassistant.core import HomeAssistant, callback
+from homeassistant.exceptions import HomeAssistantError
 from homeassistant.helpers import entity_registry as ent_reg
 from homeassistant.helpers.dispatcher import (
     async_dispatcher_send,
@@ -46,7 +47,6 @@ from .const import (
     SONOS_CREATE_BATTERY,
     SONOS_CREATE_MEDIA_PLAYER,
     SONOS_ENTITY_CREATED,
-    SONOS_GROUP_UPDATE,
     SONOS_POLL_UPDATE,
     SONOS_REBOOTED,
     SONOS_SEEN,
@@ -206,11 +206,6 @@ class SonosSpeaker:
             f"{SONOS_ENTITY_CREATED}-{self.soco.uid}",
             self.async_handle_new_entity,
         )
-        self._group_dispatcher = dispatcher_connect(
-            self.hass,
-            SONOS_GROUP_UPDATE,
-            self.async_update_groups,
-        )
         self._seen_dispatcher = dispatcher_connect(
             self.hass, f"{SONOS_SEEN}-{self.soco.uid}", self.async_seen
         )
@@ -352,7 +347,7 @@ class SonosSpeaker:
         """Cancel all subscriptions."""
         _LOGGER.debug("Unsubscribing from events for %s", self.zone_name)
         await asyncio.gather(
-            *[subscription.unsubscribe() for subscription in self._subscriptions],
+            *(subscription.unsubscribe() for subscription in self._subscriptions),
             return_exceptions=True,
         )
         self._subscriptions = []
@@ -465,7 +460,6 @@ class SonosSpeaker:
             self.soco = soco
 
         was_available = self.available
-        _LOGGER.debug("Async seen: %s, was_available: %s", self.soco, was_available)
 
         if self._seen_timer:
             self._seen_timer()
@@ -478,6 +472,12 @@ class SonosSpeaker:
             self.async_write_entity_states()
             return
 
+        _LOGGER.debug(
+            "%s [%s] was not available, setting up",
+            self.zone_name,
+            self.soco.ip_address,
+        )
+
         self._poll_timer = self.hass.helpers.event.async_track_time_interval(
             partial(
                 async_dispatcher_send,
@@ -612,22 +612,18 @@ class SonosSpeaker:
     #
     # Group management
     #
-    def update_groups(self, event: SonosEvent | None = None) -> None:
-        """Handle callback for topology change event."""
-        coro = self.create_update_groups_coro(event)
-        if coro:
-            self.hass.add_job(coro)  # type: ignore
+    def update_groups(self) -> None:
+        """Update group topology when polling."""
+        self.hass.add_job(self.create_update_groups_coro())
 
     @callback
-    def async_update_groups(self, event: SonosEvent | None = None) -> None:
+    def async_update_groups(self, event: SonosEvent) -> None:
         """Handle callback for topology change event."""
-        coro = self.create_update_groups_coro(event)
-        if coro:
-            self.hass.async_add_job(coro)  # type: ignore
+        if not hasattr(event, "zone_player_uui_ds_in_group"):
+            return None
+        self.hass.async_add_job(self.create_update_groups_coro(event))
 
-    def create_update_groups_coro(
-        self, event: SonosEvent | None = None
-    ) -> Coroutine | None:
+    def create_update_groups_coro(self, event: SonosEvent | None = None) -> Coroutine:
         """Handle callback for topology change event."""
 
         def _get_soco_group() -> list[str]:
@@ -646,7 +642,7 @@ class SonosSpeaker:
 
             return [coordinator_uid] + slave_uids
 
-        async def _async_extract_group(event: SonosEvent) -> list[str]:
+        async def _async_extract_group(event: SonosEvent | None) -> list[str]:
             """Extract group layout from a topology event."""
             group = event and event.zone_player_uui_ds_in_group
             if group:
@@ -658,6 +654,10 @@ class SonosSpeaker:
         @callback
         def _async_regroup(group: list[str]) -> None:
             """Rebuild internal group layout."""
+            if group == [self.soco.uid] and self.sonos_group == [self]:
+                # Skip updating existing single speakers in polling mode
+                return
+
             entity_registry = ent_reg.async_get(self.hass)
             sonos_group = []
             sonos_group_entities = []
@@ -671,6 +671,11 @@ class SonosSpeaker:
                     )
                     sonos_group_entities.append(entity_id)
 
+            if self.sonos_group_entities == sonos_group_entities:
+                # Useful in polling mode for speakers with stereo pairs or surrounds
+                # as those "invisible" speakers will bypass the single speaker check
+                return
+
             self.coordinator = None
             self.sonos_group = sonos_group
             self.sonos_group_entities = sonos_group_entities
@@ -684,7 +689,9 @@ class SonosSpeaker:
                     slave.sonos_group_entities = sonos_group_entities
                     slave.async_write_entity_states()
 
-        async def _async_handle_group_event(event: SonosEvent) -> None:
+            _LOGGER.debug("Regrouped %s: %s", self.zone_name, self.sonos_group_entities)
+
+        async def _async_handle_group_event(event: SonosEvent | None) -> None:
             """Get async lock and handle event."""
 
             async with self.hass.data[DATA_SONOS].topology_condition:
@@ -695,9 +702,6 @@ class SonosSpeaker:
 
                     self.hass.data[DATA_SONOS].topology_condition.notify_all()
 
-        if event and not hasattr(event, "zone_player_uui_ds_in_group"):
-            return None
-
         return _async_handle_group_event(event)
 
     @soco_error()
@@ -804,27 +808,58 @@ class SonosSpeaker:
         """Restore snapshots for all the speakers."""
 
         def _restore_groups(
-            speakers: list[SonosSpeaker], with_group: bool
+            speakers: set[SonosSpeaker], with_group: bool
         ) -> list[list[SonosSpeaker]]:
             """Pause all current coordinators and restore groups."""
             for speaker in (s for s in speakers if s.is_coordinator):
-                if speaker.media.playback_status == SONOS_STATE_PLAYING:
-                    speaker.soco.pause()
+                if (
+                    speaker.media.playback_status == SONOS_STATE_PLAYING
+                    and "Pause" in speaker.soco.available_actions
+                ):
+                    try:
+                        speaker.soco.pause()
+                    except SoCoUPnPException as exc:
+                        _LOGGER.debug(
+                            "Pause failed during restore of %s: %s",
+                            speaker.zone_name,
+                            speaker.soco.available_actions,
+                            exc_info=exc,
+                        )
 
             groups = []
+            if not with_group:
+                return groups
 
-            if with_group:
-                # Unjoin slaves first to prevent inheritance of queues
-                for speaker in [s for s in speakers if not s.is_coordinator]:
-                    if speaker.snapshot_group != speaker.sonos_group:
-                        speaker.unjoin()
+            # Unjoin non-coordinator speakers not contained in the desired snapshot group
+            #
+            # If a coordinator is unjoined from its group, another speaker from the group
+            # will inherit the coordinator's playqueue and its own playqueue will be lost
+            speakers_to_unjoin = set()
+            for speaker in speakers:
+                if speaker.sonos_group == speaker.snapshot_group:
+                    continue
 
-                # Bring back the original group topology
-                for speaker in (s for s in speakers if s.snapshot_group):
-                    assert speaker.snapshot_group is not None
-                    if speaker.snapshot_group[0] == speaker:
+                speakers_to_unjoin.update(
+                    {
+                        s
+                        for s in speaker.sonos_group[1:]
+                        if s not in speaker.snapshot_group
+                    }
+                )
+
+            for speaker in speakers_to_unjoin:
+                speaker.unjoin()
+
+            # Bring back the original group topology
+            for speaker in (s for s in speakers if s.snapshot_group):
+                assert speaker.snapshot_group is not None
+                if speaker.snapshot_group[0] == speaker:
+                    if (
+                        speaker.snapshot_group != speaker.sonos_group
+                        and speaker.snapshot_group != [speaker]
+                    ):
                         speaker.join(speaker.snapshot_group)
-                        groups.append(speaker.snapshot_group.copy())
+                    groups.append(speaker.snapshot_group.copy())
 
             return groups
 
@@ -838,6 +873,11 @@ class SonosSpeaker:
 
         # Find all affected players
         speakers_set = {s for s in speakers if s.soco_snapshot}
+        if missing_snapshots := set(speakers) - speakers_set:
+            raise HomeAssistantError(
+                f"Restore failed, speakers are missing snapshots: {[s.zone_name for s in missing_snapshots]}"
+            )
+
         if with_group:
             for speaker in [s for s in speakers_set if s.snapshot_group]:
                 assert speaker.snapshot_group is not None
