@@ -1,6 +1,8 @@
 """Support for Z-Wave lights."""
+from __future__ import annotations
+
 import logging
-from typing import Any, Callable, Dict, Optional, Tuple
+from typing import Any
 
 from zwave_js_server.client import Client as ZwaveClient
 from zwave_js_server.const import ColorComponent, CommandClass
@@ -9,22 +11,23 @@ from homeassistant.components.light import (
     ATTR_BRIGHTNESS,
     ATTR_COLOR_TEMP,
     ATTR_HS_COLOR,
+    ATTR_RGBW_COLOR,
     ATTR_TRANSITION,
-    ATTR_WHITE_VALUE,
+    COLOR_MODE_BRIGHTNESS,
+    COLOR_MODE_COLOR_TEMP,
+    COLOR_MODE_HS,
+    COLOR_MODE_RGBW,
     DOMAIN as LIGHT_DOMAIN,
-    SUPPORT_BRIGHTNESS,
-    SUPPORT_COLOR,
-    SUPPORT_COLOR_TEMP,
     SUPPORT_TRANSITION,
-    SUPPORT_WHITE_VALUE,
     LightEntity,
 )
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers.dispatcher import async_dispatcher_connect
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 import homeassistant.util.color as color_util
 
-from .const import DATA_CLIENT, DATA_UNSUBSCRIBE, DOMAIN
+from .const import DATA_CLIENT, DOMAIN
 from .discovery import ZwaveDiscoveryInfo
 from .entity import ZWaveBaseEntity
 
@@ -41,9 +44,13 @@ MULTI_COLOR_MAP = {
     ColorComponent.PURPLE: "purple",
 }
 
+TRANSITION_DURATION = "transitionDuration"
+
 
 async def async_setup_entry(
-    hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: Callable
+    hass: HomeAssistant,
+    config_entry: ConfigEntry,
+    async_add_entities: AddEntitiesCallback,
 ) -> None:
     """Set up Z-Wave Light from Config Entry."""
     client: ZwaveClient = hass.data[DOMAIN][config_entry.entry_id][DATA_CLIENT]
@@ -55,7 +62,7 @@ async def async_setup_entry(
         light = ZwaveLight(config_entry, client, info)
         async_add_entities([light])
 
-    hass.data[DOMAIN][config_entry.entry_id][DATA_UNSUBSCRIBE].append(
+    config_entry.async_on_unload(
         async_dispatcher_connect(
             hass,
             f"{DOMAIN}_{config_entry.entry_id}_add_{LIGHT_DOMAIN}",
@@ -83,27 +90,58 @@ class ZwaveLight(ZWaveBaseEntity, LightEntity):
         """Initialize the light."""
         super().__init__(config_entry, client, info)
         self._supports_color = False
-        self._supports_white_value = False
+        self._supports_rgbw = False
         self._supports_color_temp = False
-        self._hs_color: Optional[Tuple[float, float]] = None
-        self._white_value: Optional[int] = None
-        self._color_temp: Optional[int] = None
+        self._hs_color: tuple[float, float] | None = None
+        self._rgbw_color: tuple[int, int, int, int] | None = None
+        self._color_mode: str | None = None
+        self._color_temp: int | None = None
         self._min_mireds = 153  # 6500K as a safe default
         self._max_mireds = 370  # 2700K as a safe default
-        self._supported_features = SUPPORT_BRIGHTNESS
+        self._warm_white = self.get_zwave_value(
+            "targetColor",
+            CommandClass.SWITCH_COLOR,
+            value_property_key=ColorComponent.WARM_WHITE,
+        )
+        self._cold_white = self.get_zwave_value(
+            "targetColor",
+            CommandClass.SWITCH_COLOR,
+            value_property_key=ColorComponent.COLD_WHITE,
+        )
+        self._supported_color_modes = set()
 
         # get additional (optional) values and set features
-        self._target_value = self.get_zwave_value("targetValue")
-        self._dimming_duration = self.get_zwave_value("duration")
-        if self._dimming_duration is not None:
-            self._supported_features |= SUPPORT_TRANSITION
+        self._target_brightness = self.get_zwave_value(
+            "targetValue", add_to_watched_value_ids=False
+        )
+        self._target_color = self.get_zwave_value(
+            "targetColor", CommandClass.SWITCH_COLOR, add_to_watched_value_ids=False
+        )
+
         self._calculate_color_values()
-        if self._supports_color:
-            self._supported_features |= SUPPORT_COLOR
+        if self._supports_rgbw:
+            self._supported_color_modes.add(COLOR_MODE_RGBW)
+        elif self._supports_color:
+            self._supported_color_modes.add(COLOR_MODE_HS)
         if self._supports_color_temp:
-            self._supported_features |= SUPPORT_COLOR_TEMP
-        if self._supports_white_value:
-            self._supported_features |= SUPPORT_WHITE_VALUE
+            self._supported_color_modes.add(COLOR_MODE_COLOR_TEMP)
+        if not self._supported_color_modes:
+            self._supported_color_modes.add(COLOR_MODE_BRIGHTNESS)
+
+        # Entity class attributes
+        self._attr_supported_features = 0
+        self.supports_brightness_transition = bool(
+            self._target_brightness is not None
+            and TRANSITION_DURATION
+            in self._target_brightness.metadata.value_change_options
+        )
+        self.supports_color_transition = bool(
+            self._target_color is not None
+            and TRANSITION_DURATION in self._target_color.metadata.value_change_options
+        )
+
+        if self.supports_brightness_transition or self.supports_color_transition:
+            self._attr_supported_features |= SUPPORT_TRANSITION
 
     @callback
     def on_value_update(self) -> None:
@@ -120,23 +158,28 @@ class ZwaveLight(ZWaveBaseEntity, LightEntity):
             return round((self.info.primary_value.value / 99) * 255)
         return 0
 
+    @property
+    def color_mode(self) -> str | None:
+        """Return the color mode of the light."""
+        return self._color_mode
+
     @property
     def is_on(self) -> bool:
         """Return true if device is on (brightness above 0)."""
         return self.brightness > 0
 
     @property
-    def hs_color(self) -> Optional[Tuple[float, float]]:
+    def hs_color(self) -> tuple[float, float] | None:
         """Return the hs color."""
         return self._hs_color
 
     @property
-    def white_value(self) -> Optional[int]:
-        """Return the white value of this light between 0..255."""
-        return self._white_value
+    def rgbw_color(self) -> tuple[int, int, int, int] | None:
+        """Return the hs color."""
+        return self._rgbw_color
 
     @property
-    def color_temp(self) -> Optional[int]:
+    def color_temp(self) -> int | None:
         """Return the color temperature."""
         return self._color_temp
 
@@ -151,12 +194,15 @@ class ZwaveLight(ZWaveBaseEntity, LightEntity):
         return self._max_mireds
 
     @property
-    def supported_features(self) -> Optional[int]:
+    def supported_color_modes(self) -> set | None:
         """Flag supported features."""
-        return self._supported_features
+        return self._supported_color_modes
 
     async def async_turn_on(self, **kwargs: Any) -> None:
         """Turn the device on."""
+
+        transition = kwargs.get(ATTR_TRANSITION)
+
         # RGB/HS color
         hs_color = kwargs.get(ATTR_HS_COLOR)
         if hs_color is not None and self._supports_color:
@@ -170,7 +216,7 @@ class ZwaveLight(ZWaveBaseEntity, LightEntity):
                 # turn of white leds when setting rgb
                 colors[ColorComponent.WARM_WHITE] = 0
                 colors[ColorComponent.COLD_WHITE] = 0
-            await self._async_set_colors(colors)
+            await self._async_set_colors(colors, transition)
 
         # Color temperature
         color_temp = kwargs.get(ATTR_COLOR_TEMP)
@@ -196,31 +242,35 @@ class ZwaveLight(ZWaveBaseEntity, LightEntity):
                     ColorComponent.BLUE: 0,
                     ColorComponent.WARM_WHITE: warm,
                     ColorComponent.COLD_WHITE: cold,
-                }
+                },
+                transition,
             )
 
-        # White value
-        white_value = kwargs.get(ATTR_WHITE_VALUE)
-        if white_value is not None and self._supports_white_value:
-            # white led brightness is controlled by white level
-            # rgb leds (if any) can be on at the same time
-            await self._async_set_colors(
-                {
-                    ColorComponent.WARM_WHITE: white_value,
-                    ColorComponent.COLD_WHITE: white_value,
-                }
-            )
+        # RGBW
+        rgbw = kwargs.get(ATTR_RGBW_COLOR)
+        if rgbw is not None and self._supports_rgbw:
+            rgbw_channels = {
+                ColorComponent.RED: rgbw[0],
+                ColorComponent.GREEN: rgbw[1],
+                ColorComponent.BLUE: rgbw[2],
+            }
+            if self._warm_white:
+                rgbw_channels[ColorComponent.WARM_WHITE] = rgbw[3]
+
+            if self._cold_white:
+                rgbw_channels[ColorComponent.COLD_WHITE] = rgbw[3]
+            await self._async_set_colors(rgbw_channels, transition)
 
         # set brightness
-        await self._async_set_brightness(
-            kwargs.get(ATTR_BRIGHTNESS), kwargs.get(ATTR_TRANSITION)
-        )
+        await self._async_set_brightness(kwargs.get(ATTR_BRIGHTNESS), transition)
 
     async def async_turn_off(self, **kwargs: Any) -> None:
         """Turn the light off."""
         await self._async_set_brightness(0, kwargs.get(ATTR_TRANSITION))
 
-    async def _async_set_colors(self, colors: Dict[ColorComponent, int]) -> None:
+    async def _async_set_colors(
+        self, colors: dict[ColorComponent, int], transition: float | None = None
+    ) -> None:
         """Set (multiple) defined colors to given value(s)."""
         # prefer the (new) combined color property
         # https://github.com/zwave-js/node-zwave-js/pull/1782
@@ -229,36 +279,50 @@ class ZwaveLight(ZWaveBaseEntity, LightEntity):
             CommandClass.SWITCH_COLOR,
             value_property_key=None,
         )
+        zwave_transition = None
+
+        if self.supports_color_transition:
+            if transition is not None:
+                zwave_transition = {TRANSITION_DURATION: f"{int(transition)}s"}
+            else:
+                zwave_transition = {TRANSITION_DURATION: "default"}
+
         if combined_color_val and isinstance(combined_color_val.value, dict):
             colors_dict = {}
             for color, value in colors.items():
                 color_name = MULTI_COLOR_MAP[color]
                 colors_dict[color_name] = value
             # set updated color object
-            await self.info.node.async_set_value(combined_color_val, colors_dict)
+            await self.info.node.async_set_value(
+                combined_color_val, colors_dict, zwave_transition
+            )
             return
 
         # fallback to setting the color(s) one by one if multicolor fails
         # not sure this is needed at all, but just in case
         for color, value in colors.items():
-            await self._async_set_color(color, value)
+            await self._async_set_color(color, value, zwave_transition)
 
-    async def _async_set_color(self, color: ColorComponent, new_value: int) -> None:
+    async def _async_set_color(
+        self,
+        color: ColorComponent,
+        new_value: int,
+        transition: dict[str, str] | None = None,
+    ) -> None:
         """Set defined color to given value."""
-        property_key = color.value
         # actually set the new color value
         target_zwave_value = self.get_zwave_value(
             "targetColor",
             CommandClass.SWITCH_COLOR,
-            value_property_key=property_key.key,
+            value_property_key=color.value,
         )
         if target_zwave_value is None:
             # guard for unsupported color
             return
-        await self.info.node.async_set_value(target_zwave_value, new_value)
+        await self.info.node.async_set_value(target_zwave_value, new_value, transition)
 
     async def _async_set_brightness(
-        self, brightness: Optional[int], transition: Optional[int] = None
+        self, brightness: int | None, transition: float | None = None
     ) -> None:
         """Set new brightness to light."""
         if brightness is None:
@@ -269,42 +333,17 @@ class ZwaveLight(ZWaveBaseEntity, LightEntity):
             zwave_brightness = byte_to_zwave_brightness(brightness)
 
         # set transition value before sending new brightness
-        await self._async_set_transition_duration(transition)
-        # setting a value requires setting targetValue
-        await self.info.node.async_set_value(self._target_value, zwave_brightness)
-
-    async def _async_set_transition_duration(
-        self, duration: Optional[int] = None
-    ) -> None:
-        """Set the transition time for the brightness value."""
-        if self._dimming_duration is None:
-            return
-        # pylint: disable=fixme,unreachable
-        # TODO: setting duration needs to be fixed upstream
-        # https://github.com/zwave-js/node-zwave-js/issues/1321
-        return
-
-        if duration is None:  # type: ignore
-            # no transition specified by user, use defaults
-            duration = 7621  # anything over 7620 uses the factory default
-        else:  # pragma: no cover
-            # transition specified by user
-            transition = duration
-            if transition <= 127:
-                duration = transition
+        zwave_transition = None
+        if self.supports_brightness_transition:
+            if transition is not None:
+                zwave_transition = {TRANSITION_DURATION: f"{int(transition)}s"}
             else:
-                minutes = round(transition / 60)
-                LOGGER.debug(
-                    "Transition rounded to %d minutes for %s",
-                    minutes,
-                    self.entity_id,
-                )
-                duration = minutes + 128
+                zwave_transition = {TRANSITION_DURATION: "default"}
 
-        # only send value if it differs from current
-        # this prevents sending a command for nothing
-        if self._dimming_duration.value != duration:  # pragma: no cover
-            await self.info.node.async_set_value(self._dimming_duration, duration)
+        # setting a value requires setting targetValue
+        await self.info.node.async_set_value(
+            self._target_brightness, zwave_brightness, zwave_transition
+        )
 
     @callback
     def _calculate_color_values(self) -> None:
@@ -315,27 +354,27 @@ class ZwaveLight(ZWaveBaseEntity, LightEntity):
         red_val = self.get_zwave_value(
             "currentColor",
             CommandClass.SWITCH_COLOR,
-            value_property_key=ColorComponent.RED.value.key,
+            value_property_key=ColorComponent.RED.value,
         )
         green_val = self.get_zwave_value(
             "currentColor",
             CommandClass.SWITCH_COLOR,
-            value_property_key=ColorComponent.GREEN.value.key,
+            value_property_key=ColorComponent.GREEN.value,
         )
         blue_val = self.get_zwave_value(
             "currentColor",
             CommandClass.SWITCH_COLOR,
-            value_property_key=ColorComponent.BLUE.value.key,
+            value_property_key=ColorComponent.BLUE.value,
         )
         ww_val = self.get_zwave_value(
             "currentColor",
             CommandClass.SWITCH_COLOR,
-            value_property_key=ColorComponent.WARM_WHITE.value.key,
+            value_property_key=ColorComponent.WARM_WHITE.value,
         )
         cw_val = self.get_zwave_value(
             "currentColor",
             CommandClass.SWITCH_COLOR,
-            value_property_key=ColorComponent.COLD_WHITE.value.key,
+            value_property_key=ColorComponent.COLD_WHITE.value,
         )
         # prefer the (new) combined color property
         # https://github.com/zwave-js/node-zwave-js/pull/1782
@@ -349,6 +388,9 @@ class ZwaveLight(ZWaveBaseEntity, LightEntity):
         else:
             multi_color = {}
 
+        # Default: Brightness (no color)
+        self._color_mode = COLOR_MODE_BRIGHTNESS
+
         # RGB support
         if red_val and green_val and blue_val:
             # prefer values from the multicolor property
@@ -356,8 +398,11 @@ class ZwaveLight(ZWaveBaseEntity, LightEntity):
             green = multi_color.get("green", green_val.value)
             blue = multi_color.get("blue", blue_val.value)
             self._supports_color = True
-            # convert to HS
-            self._hs_color = color_util.color_RGB_to_hs(red, green, blue)
+            if None not in (red, green, blue):
+                # convert to HS
+                self._hs_color = color_util.color_RGB_to_hs(red, green, blue)
+                # Light supports color, set color mode to hs
+                self._color_mode = COLOR_MODE_HS
 
         # color temperature support
         if ww_val and cw_val:
@@ -370,13 +415,21 @@ class ZwaveLight(ZWaveBaseEntity, LightEntity):
                     self._max_mireds
                     - ((cold_white / 255) * (self._max_mireds - self._min_mireds))
                 )
+                # White channels turned on, set color mode to color_temp
+                self._color_mode = COLOR_MODE_COLOR_TEMP
             else:
                 self._color_temp = None
-        # only one white channel (warm white) = white_level support
-        elif ww_val:
-            self._supports_white_value = True
-            self._white_value = multi_color.get("warmWhite", ww_val.value)
-        # only one white channel (cool white) = white_level support
+        # only one white channel (warm white) = rgbw support
+        elif red_val and green_val and blue_val and ww_val:
+            self._supports_rgbw = True
+            white = multi_color.get("warmWhite", ww_val.value)
+            self._rgbw_color = (red, green, blue, white)
+            # Light supports rgbw, set color mode to rgbw
+            self._color_mode = COLOR_MODE_RGBW
+        # only one white channel (cool white) = rgbw support
         elif cw_val:
-            self._supports_white_value = True
-            self._white_value = multi_color.get("coldWhite", cw_val.value)
+            self._supports_rgbw = True
+            white = multi_color.get("coldWhite", cw_val.value)
+            self._rgbw_color = (red, green, blue, white)
+            # Light supports rgbw, set color mode to rgbw
+            self._color_mode = COLOR_MODE_RGBW
