@@ -1,14 +1,17 @@
 """Module to help with parsing and generating configuration files."""
-# pylint: disable=no-name-in-module
+from __future__ import annotations
+
 from collections import OrderedDict
-from distutils.version import LooseVersion  # pylint: disable=import-error
+from collections.abc import Sequence
 import logging
 import os
+from pathlib import Path
 import re
 import shutil
 from types import ModuleType
-from typing import Any, Callable, Dict, Optional, Sequence, Set, Tuple, Union
+from typing import Any, Callable
 
+from awesomeversion import AwesomeVersion
 import voluptuous as vol
 from voluptuous.humanize import humanize_error
 
@@ -21,15 +24,22 @@ from homeassistant.const import (
     ATTR_ASSUMED_STATE,
     ATTR_FRIENDLY_NAME,
     ATTR_HIDDEN,
+    CONF_ALLOWLIST_EXTERNAL_DIRS,
+    CONF_ALLOWLIST_EXTERNAL_URLS,
     CONF_AUTH_MFA_MODULES,
     CONF_AUTH_PROVIDERS,
+    CONF_CURRENCY,
     CONF_CUSTOMIZE,
     CONF_CUSTOMIZE_DOMAIN,
     CONF_CUSTOMIZE_GLOB,
     CONF_ELEVATION,
+    CONF_EXTERNAL_URL,
     CONF_ID,
+    CONF_INTERNAL_URL,
     CONF_LATITUDE,
+    CONF_LEGACY_TEMPLATES,
     CONF_LONGITUDE,
+    CONF_MEDIA_DIRS,
     CONF_NAME,
     CONF_PACKAGES,
     CONF_TEMPERATURE_UNIT,
@@ -37,7 +47,7 @@ from homeassistant.const import (
     CONF_TYPE,
     CONF_UNIT_SYSTEM,
     CONF_UNIT_SYSTEM_IMPERIAL,
-    CONF_WHITELIST_EXTERNAL_DIRS,
+    LEGACY_CONF_WHITELIST_EXTERNAL_DIRS,
     TEMP_CELSIUS,
     __version__,
 )
@@ -46,6 +56,7 @@ from homeassistant.exceptions import HomeAssistantError
 from homeassistant.helpers import config_per_platform, extract_domain_configs
 import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers.entity_values import EntityValues
+from homeassistant.helpers.typing import ConfigType
 from homeassistant.loader import Integration, IntegrationNotFound
 from homeassistant.requirements import (
     RequirementsNotFound,
@@ -53,7 +64,7 @@ from homeassistant.requirements import (
 )
 from homeassistant.util.package import is_docker_env
 from homeassistant.util.unit_system import IMPERIAL_SYSTEM, METRIC_SYSTEM
-from homeassistant.util.yaml import SECRET_YAML, load_yaml
+from homeassistant.util.yaml import SECRET_YAML, Secrets, load_yaml
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -70,14 +81,17 @@ AUTOMATION_CONFIG_PATH = "automations.yaml"
 SCRIPT_CONFIG_PATH = "scripts.yaml"
 SCENE_CONFIG_PATH = "scenes.yaml"
 
+LOAD_EXCEPTIONS = (ImportError, FileNotFoundError)
+INTEGRATION_LOAD_EXCEPTIONS = (
+    IntegrationNotFound,
+    RequirementsNotFound,
+    *LOAD_EXCEPTIONS,
+)
+
 DEFAULT_CONFIG = f"""
 # Configure a default setup of Home Assistant (frontend, api, etc)
 default_config:
 
-# Uncomment this if you are using SSL/TLS, running in Docker container, etc.
-# http:
-#   base_url: example.duckdns.org:8123
-
 # Text to speech
 tts:
   - platform: google_translate
@@ -89,7 +103,7 @@ scene: !include {SCENE_CONFIG_PATH}
 """
 DEFAULT_SECRETS = """
 # Use this file to store secrets like usernames and passwords.
-# Learn more at https://home-assistant.io/docs/configuration/secrets/
+# Learn more at https://www.home-assistant.io/docs/configuration/secrets/
 some_password: welcome
 """
 TTS_PRE_92 = """
@@ -104,30 +118,29 @@ tts:
 
 
 def _no_duplicate_auth_provider(
-    configs: Sequence[Dict[str, Any]]
-) -> Sequence[Dict[str, Any]]:
+    configs: Sequence[dict[str, Any]]
+) -> Sequence[dict[str, Any]]:
     """No duplicate auth provider config allowed in a list.
 
     Each type of auth provider can only have one config without optional id.
     Unique id is required if same type of auth provider used multiple times.
     """
-    config_keys: Set[Tuple[str, Optional[str]]] = set()
+    config_keys: set[tuple[str, str | None]] = set()
     for config in configs:
         key = (config[CONF_TYPE], config.get(CONF_ID))
         if key in config_keys:
             raise vol.Invalid(
-                "Duplicate auth provider {} found. Please add unique IDs if "
-                "you want to have the same auth provider twice".format(
-                    config[CONF_TYPE]
-                )
+                f"Duplicate auth provider {config[CONF_TYPE]} found. "
+                "Please add unique IDs "
+                "if you want to have the same auth provider twice"
             )
         config_keys.add(key)
     return configs
 
 
 def _no_duplicate_auth_mfa_module(
-    configs: Sequence[Dict[str, Any]]
-) -> Sequence[Dict[str, Any]]:
+    configs: Sequence[dict[str, Any]]
+) -> Sequence[dict[str, Any]]:
     """No duplicate auth mfa module item allowed in a list.
 
     Each type of mfa module can only have one config without optional id.
@@ -135,13 +148,14 @@ def _no_duplicate_auth_mfa_module(
     times.
     Note: this is different than auth provider
     """
-    config_keys: Set[str] = set()
+    config_keys: set[str] = set()
     for config in configs:
         key = config.get(CONF_ID, config[CONF_TYPE])
         if key in config_keys:
             raise vol.Invalid(
-                "Duplicate mfa module {} found. Please add unique IDs if "
-                "you want to have the same mfa module twice".format(config[CONF_TYPE])
+                f"Duplicate mfa module {config[CONF_TYPE]} found. "
+                "Please add unique IDs "
+                "if you want to have the same mfa module twice"
             )
         config_keys.add(key)
     return configs
@@ -183,9 +197,15 @@ CORE_CONFIG_SCHEMA = CUSTOMIZE_CONFIG_SCHEMA.extend(
         vol.Optional(CONF_TEMPERATURE_UNIT): cv.temperature_unit,
         CONF_UNIT_SYSTEM: cv.unit_system,
         CONF_TIME_ZONE: cv.time_zone,
-        vol.Optional(CONF_WHITELIST_EXTERNAL_DIRS):
-        # pylint: disable=no-value-for-parameter
-        vol.All(cv.ensure_list, [vol.IsDir()]),
+        vol.Optional(CONF_INTERNAL_URL): cv.url,
+        vol.Optional(CONF_EXTERNAL_URL): cv.url,
+        vol.Optional(CONF_ALLOWLIST_EXTERNAL_DIRS): vol.All(
+            cv.ensure_list, [vol.IsDir()]  # pylint: disable=no-value-for-parameter
+        ),
+        vol.Optional(LEGACY_CONF_WHITELIST_EXTERNAL_DIRS): vol.All(
+            cv.ensure_list, [vol.IsDir()]  # pylint: disable=no-value-for-parameter
+        ),
+        vol.Optional(CONF_ALLOWLIST_EXTERNAL_URLS): vol.All(cv.ensure_list, [cv.url]),
         vol.Optional(CONF_PACKAGES, default={}): PACKAGES_CONFIG_SCHEMA,
         vol.Optional(CONF_AUTH_PROVIDERS): vol.All(
             cv.ensure_list,
@@ -216,6 +236,10 @@ CORE_CONFIG_SCHEMA = CUSTOMIZE_CONFIG_SCHEMA.extend(
             ],
             _no_duplicate_auth_mfa_module,
         ),
+        # pylint: disable=no-value-for-parameter
+        vol.Optional(CONF_MEDIA_DIRS): cv.schema_with_slug_keys(vol.IsDir()),
+        vol.Optional(CONF_LEGACY_TEMPLATES): cv.boolean,
+        vol.Optional(CONF_CURRENCY): cv.currency,
     }
 )
 
@@ -266,26 +290,31 @@ def _write_default_config(config_dir: str) -> bool:
     # Writing files with YAML does not create the most human readable results
     # So we're hard coding a YAML template.
     try:
-        with open(config_path, "wt") as config_file:
+        with open(config_path, "wt", encoding="utf8") as config_file:
             config_file.write(DEFAULT_CONFIG)
 
-        with open(secret_path, "wt") as secret_file:
-            secret_file.write(DEFAULT_SECRETS)
+        if not os.path.isfile(secret_path):
+            with open(secret_path, "wt", encoding="utf8") as secret_file:
+                secret_file.write(DEFAULT_SECRETS)
 
-        with open(version_path, "wt") as version_file:
+        with open(version_path, "wt", encoding="utf8") as version_file:
             version_file.write(__version__)
 
-        with open(group_yaml_path, "wt"):
-            pass
+        if not os.path.isfile(group_yaml_path):
+            with open(group_yaml_path, "wt", encoding="utf8"):
+                pass
 
-        with open(automation_yaml_path, "wt") as fil:
-            fil.write("[]")
+        if not os.path.isfile(automation_yaml_path):
+            with open(automation_yaml_path, "wt", encoding="utf8") as automation_file:
+                automation_file.write("[]")
 
-        with open(script_yaml_path, "wt"):
-            pass
+        if not os.path.isfile(script_yaml_path):
+            with open(script_yaml_path, "wt", encoding="utf8"):
+                pass
 
-        with open(scene_yaml_path, "wt"):
-            pass
+        if not os.path.isfile(scene_yaml_path):
+            with open(scene_yaml_path, "wt", encoding="utf8"):
+                pass
 
         return True
 
@@ -294,33 +323,44 @@ def _write_default_config(config_dir: str) -> bool:
         return False
 
 
-async def async_hass_config_yaml(hass: HomeAssistant) -> Dict:
+async def async_hass_config_yaml(hass: HomeAssistant) -> dict:
     """Load YAML from a Home Assistant configuration file.
 
     This function allow a component inside the asyncio loop to reload its
     configuration by itself. Include package merge.
     """
+    if hass.config.config_dir is None:
+        secrets = None
+    else:
+        secrets = Secrets(Path(hass.config.config_dir))
+
     # Not using async_add_executor_job because this is an internal method.
     config = await hass.loop.run_in_executor(
-        None, load_yaml_config_file, hass.config.path(YAML_CONFIG_FILE)
+        None,
+        load_yaml_config_file,
+        hass.config.path(YAML_CONFIG_FILE),
+        secrets,
     )
     core_config = config.get(CONF_CORE, {})
     await merge_packages_config(hass, config, core_config.get(CONF_PACKAGES, {}))
     return config
 
 
-def load_yaml_config_file(config_path: str) -> Dict[Any, Any]:
+def load_yaml_config_file(
+    config_path: str, secrets: Secrets | None = None
+) -> dict[Any, Any]:
     """Parse a YAML configuration file.
 
     Raises FileNotFoundError or HomeAssistantError.
 
     This method needs to run in an executor.
     """
-    conf_dict = load_yaml(config_path)
+    conf_dict = load_yaml(config_path, secrets)
 
     if not isinstance(conf_dict, dict):
-        msg = "The configuration file {} does not contain a dictionary".format(
-            os.path.basename(config_path)
+        msg = (
+            f"The configuration file {os.path.basename(config_path)} "
+            "does not contain a dictionary"
         )
         _LOGGER.error(msg)
         raise HomeAssistantError(msg)
@@ -339,7 +379,7 @@ def process_ha_config_upgrade(hass: HomeAssistant) -> None:
     version_path = hass.config.path(VERSION_FILE)
 
     try:
-        with open(version_path, "rt") as inp:
+        with open(version_path, encoding="utf8") as inp:
             conf_version = inp.readline().strip()
     except FileNotFoundError:
         # Last version to not have this file
@@ -352,19 +392,19 @@ def process_ha_config_upgrade(hass: HomeAssistant) -> None:
         "Upgrading configuration directory from %s to %s", conf_version, __version__
     )
 
-    version_obj = LooseVersion(conf_version)
+    version_obj = AwesomeVersion(conf_version)
 
-    if version_obj < LooseVersion("0.50"):
+    if version_obj < AwesomeVersion("0.50"):
         # 0.50 introduced persistent deps dir.
         lib_path = hass.config.path("deps")
         if os.path.isdir(lib_path):
             shutil.rmtree(lib_path)
 
-    if version_obj < LooseVersion("0.92"):
+    if version_obj < AwesomeVersion("0.92"):
         # 0.92 moved google/tts.py to google_translate/tts.py
         config_path = hass.config.path(YAML_CONFIG_FILE)
 
-        with open(config_path, "rt", encoding="utf-8") as config_file:
+        with open(config_path, encoding="utf-8") as config_file:
             config_raw = config_file.read()
 
         if TTS_PRE_92 in config_raw:
@@ -375,16 +415,15 @@ def process_ha_config_upgrade(hass: HomeAssistant) -> None:
                     config_file.write(config_raw)
             except OSError:
                 _LOGGER.exception("Migrating to google_translate tts failed")
-                pass
 
-    if version_obj < LooseVersion("0.94") and is_docker_env():
+    if version_obj < AwesomeVersion("0.94") and is_docker_env():
         # In 0.94 we no longer install packages inside the deps folder when
         # running inside a Docker container.
         lib_path = hass.config.path("deps")
         if os.path.isdir(lib_path):
             shutil.rmtree(lib_path)
 
-    with open(version_path, "wt") as outp:
+    with open(version_path, "wt", encoding="utf8") as outp:
         outp.write(__version__)
 
 
@@ -392,9 +431,9 @@ def process_ha_config_upgrade(hass: HomeAssistant) -> None:
 def async_log_exception(
     ex: Exception,
     domain: str,
-    config: Dict,
+    config: dict,
     hass: HomeAssistant,
-    link: Optional[str] = None,
+    link: str | None = None,
 ) -> None:
     """Log an error for configuration validation.
 
@@ -402,50 +441,50 @@ def async_log_exception(
     """
     if hass is not None:
         async_notify_setup_error(hass, domain, link)
-    _LOGGER.error(_format_config_error(ex, domain, config, link))
+    message, is_friendly = _format_config_error(ex, domain, config, link)
+    _LOGGER.error(message, exc_info=not is_friendly and ex)
 
 
 @callback
 def _format_config_error(
-    ex: Exception, domain: str, config: Dict, link: Optional[str] = None
-) -> str:
+    ex: Exception, domain: str, config: dict, link: str | None = None
+) -> tuple[str, bool]:
     """Generate log exception for configuration validation.
 
     This method must be run in the event loop.
     """
+    is_friendly = False
     message = f"Invalid config for [{domain}]: "
     if isinstance(ex, vol.Invalid):
         if "extra keys not allowed" in ex.error_message:
+            path = "->".join(str(m) for m in ex.path)
             message += (
-                "[{option}] is an invalid option for [{domain}]. "
-                "Check: {domain}->{path}.".format(
-                    option=ex.path[-1],
-                    domain=domain,
-                    path="->".join(str(m) for m in ex.path),
-                )
+                f"[{ex.path[-1]}] is an invalid option for [{domain}]. "
+                f"Check: {domain}->{path}."
             )
         else:
-            message += "{}.".format(humanize_error(config, ex))
+            message += f"{humanize_error(config, ex)}."
+        is_friendly = True
     else:
-        message += str(ex)
+        message += str(ex) or repr(ex)
 
     try:
         domain_config = config.get(domain, config)
     except AttributeError:
         domain_config = config
 
-    message += " (See {}, line {}). ".format(
-        getattr(domain_config, "__config_file__", "?"),
-        getattr(domain_config, "__line__", "?"),
+    message += (
+        f" (See {getattr(domain_config, '__config_file__', '?')}, "
+        f"line {getattr(domain_config, '__line__', '?')}). "
     )
 
     if domain != CONF_CORE and link:
         message += f"Please check the docs at {link}"
 
-    return message
+    return message, is_friendly
 
 
-async def async_process_ha_core_config(hass: HomeAssistant, config: Dict) -> None:
+async def async_process_ha_core_config(hass: HomeAssistant, config: dict) -> None:
     """Process the [homeassistant] section from the configuration.
 
     This method is a coroutine.
@@ -473,17 +512,18 @@ async def async_process_ha_core_config(hass: HomeAssistant, config: Dict) -> Non
     hac = hass.config
 
     if any(
-        [
-            k in config
-            for k in [
-                CONF_LATITUDE,
-                CONF_LONGITUDE,
-                CONF_NAME,
-                CONF_ELEVATION,
-                CONF_TIME_ZONE,
-                CONF_UNIT_SYSTEM,
-            ]
-        ]
+        k in config
+        for k in (
+            CONF_LATITUDE,
+            CONF_LONGITUDE,
+            CONF_NAME,
+            CONF_ELEVATION,
+            CONF_TIME_ZONE,
+            CONF_UNIT_SYSTEM,
+            CONF_EXTERNAL_URL,
+            CONF_INTERNAL_URL,
+            CONF_CURRENCY,
+        )
     ):
         hac.config_source = SOURCE_YAML
 
@@ -492,6 +532,11 @@ async def async_process_ha_core_config(hass: HomeAssistant, config: Dict) -> Non
         (CONF_LONGITUDE, "longitude"),
         (CONF_NAME, "location_name"),
         (CONF_ELEVATION, "elevation"),
+        (CONF_INTERNAL_URL, "internal_url"),
+        (CONF_EXTERNAL_URL, "external_url"),
+        (CONF_MEDIA_DIRS, "media_dirs"),
+        (CONF_LEGACY_TEMPLATES, "legacy_templates"),
+        (CONF_CURRENCY, "currency"),
     ):
         if key in config:
             setattr(hac, attr, config[key])
@@ -499,10 +544,34 @@ async def async_process_ha_core_config(hass: HomeAssistant, config: Dict) -> Non
     if CONF_TIME_ZONE in config:
         hac.set_time_zone(config[CONF_TIME_ZONE])
 
+    if CONF_MEDIA_DIRS not in config:
+        if is_docker_env():
+            hac.media_dirs = {"local": "/media"}
+        else:
+            hac.media_dirs = {"local": hass.config.path("media")}
+
     # Init whitelist external dir
-    hac.whitelist_external_dirs = {hass.config.path("www")}
-    if CONF_WHITELIST_EXTERNAL_DIRS in config:
-        hac.whitelist_external_dirs.update(set(config[CONF_WHITELIST_EXTERNAL_DIRS]))
+    hac.allowlist_external_dirs = {hass.config.path("www"), *hac.media_dirs.values()}
+    if CONF_ALLOWLIST_EXTERNAL_DIRS in config:
+        hac.allowlist_external_dirs.update(set(config[CONF_ALLOWLIST_EXTERNAL_DIRS]))
+
+    elif LEGACY_CONF_WHITELIST_EXTERNAL_DIRS in config:
+        _LOGGER.warning(
+            "Key %s has been replaced with %s. Please update your config",
+            LEGACY_CONF_WHITELIST_EXTERNAL_DIRS,
+            CONF_ALLOWLIST_EXTERNAL_DIRS,
+        )
+        hac.allowlist_external_dirs.update(
+            set(config[LEGACY_CONF_WHITELIST_EXTERNAL_DIRS])
+        )
+
+    # Init whitelist external URL list – make sure to add / to every URL that doesn't
+    # already have it so that we can properly test "path ownership"
+    if CONF_ALLOWLIST_EXTERNAL_URLS in config:
+        hac.allowlist_external_urls.update(
+            url if url.endswith("/") else f"{url}/"
+            for url in config[CONF_ALLOWLIST_EXTERNAL_URLS]
+        )
 
     # Customize
     cust_exact = dict(config[CONF_CUSTOMIZE])
@@ -534,10 +603,7 @@ async def async_process_ha_core_config(hass: HomeAssistant, config: Dict) -> Non
             hac.units = METRIC_SYSTEM
     elif CONF_TEMPERATURE_UNIT in config:
         unit = config[CONF_TEMPERATURE_UNIT]
-        if unit == TEMP_CELSIUS:
-            hac.units = METRIC_SYSTEM
-        else:
-            hac.units = IMPERIAL_SYSTEM
+        hac.units = METRIC_SYSTEM if unit == TEMP_CELSIUS else IMPERIAL_SYSTEM
         _LOGGER.warning(
             "Found deprecated temperature unit in core "
             "configuration expected unit system. Replace '%s: %s' "
@@ -549,36 +615,70 @@ async def async_process_ha_core_config(hass: HomeAssistant, config: Dict) -> Non
         )
 
 
-def _log_pkg_error(package: str, component: str, config: Dict, message: str) -> None:
+def _log_pkg_error(package: str, component: str, config: dict, message: str) -> None:
     """Log an error while merging packages."""
     message = f"Package {package} setup failed. Integration {component} {message}"
 
     pack_config = config[CONF_CORE][CONF_PACKAGES].get(package, config)
-    message += " (See {}:{}). ".format(
-        getattr(pack_config, "__config_file__", "?"),
-        getattr(pack_config, "__line__", "?"),
+    message += (
+        f" (See {getattr(pack_config, '__config_file__', '?')}:"
+        f"{getattr(pack_config, '__line__', '?')}). "
     )
 
     _LOGGER.error(message)
 
 
-def _identify_config_schema(module: ModuleType) -> Tuple[Optional[str], Optional[Dict]]:
+def _identify_config_schema(module: ModuleType) -> str | None:
     """Extract the schema and identify list or dict based."""
+    if not isinstance(module.CONFIG_SCHEMA, vol.Schema):  # type: ignore
+        return None
+
+    schema = module.CONFIG_SCHEMA.schema  # type: ignore
+
+    if isinstance(schema, vol.All):
+        for subschema in schema.validators:
+            if isinstance(subschema, dict):
+                schema = subschema
+                break
+        else:
+            return None
+
     try:
-        schema = module.CONFIG_SCHEMA.schema[module.DOMAIN]  # type: ignore
-    except (AttributeError, KeyError):
-        return None, None
-    t_schema = str(schema)
+        key = next(k for k in schema if k == module.DOMAIN)  # type: ignore
+    except (TypeError, AttributeError, StopIteration):
+        return None
+    except Exception:  # pylint: disable=broad-except
+        _LOGGER.exception("Unexpected error identifying config schema")
+        return None
+
+    if hasattr(key, "default") and not isinstance(
+        key.default, vol.schema_builder.Undefined
+    ):
+        default_value = module.CONFIG_SCHEMA({module.DOMAIN: key.default()})[  # type: ignore
+            module.DOMAIN  # type: ignore
+        ]
+
+        if isinstance(default_value, dict):
+            return "dict"
+
+        if isinstance(default_value, list):
+            return "list"
+
+        return None
+
+    domain_schema = schema[key]
+
+    t_schema = str(domain_schema)
     if t_schema.startswith("{") or "schema_with_slug_keys" in t_schema:
-        return ("dict", schema)
+        return "dict"
     if t_schema.startswith(("[", "All(<function ensure_list")):
-        return ("list", schema)
-    return "", schema
+        return "list"
+    return None
 
 
-def _recursive_merge(conf: Dict[str, Any], package: Dict[str, Any]) -> Union[bool, str]:
+def _recursive_merge(conf: dict[str, Any], package: dict[str, Any]) -> bool | str:
     """Merge package into conf, recursively."""
-    error: Union[bool, str] = False
+    error: bool | str = False
     for key, pack_conf in package.items():
         if isinstance(pack_conf, dict):
             if not pack_conf:
@@ -600,10 +700,10 @@ def _recursive_merge(conf: Dict[str, Any], package: Dict[str, Any]) -> Union[boo
 
 async def merge_packages_config(
     hass: HomeAssistant,
-    config: Dict,
-    packages: Dict[str, Any],
+    config: dict,
+    packages: dict[str, Any],
     _log_pkg_error: Callable = _log_pkg_error,
-) -> Dict:
+) -> dict:
     """Merge packages into the top-level configuration. Mutate config."""
     PACKAGES_CONFIG_SCHEMA(packages)
     for pack_name, pack_conf in packages.items():
@@ -619,15 +719,14 @@ async def merge_packages_config(
                     hass, domain
                 )
                 component = integration.get_component()
-            except (IntegrationNotFound, RequirementsNotFound, ImportError) as ex:
+            except INTEGRATION_LOAD_EXCEPTIONS as ex:
                 _log_pkg_error(pack_name, comp_name, config, str(ex))
                 continue
 
             merge_list = hasattr(component, "PLATFORM_SCHEMA")
 
             if not merge_list and hasattr(component, "CONFIG_SCHEMA"):
-                merge_type, _ = _identify_config_schema(component)
-                merge_list = merge_type == "list"
+                merge_list = _identify_config_schema(component) == "list"
 
             if merge_list:
                 config[comp_name] = cv.remove_falsy(
@@ -665,9 +764,9 @@ async def merge_packages_config(
     return config
 
 
-async def async_process_component_config(
-    hass: HomeAssistant, config: Dict, integration: Integration
-) -> Optional[Dict]:
+async def async_process_component_config(  # noqa: C901
+    hass: HomeAssistant, config: ConfigType, integration: Integration
+) -> ConfigType | None:
     """Check component configuration and return processed configuration.
 
     Returns None on error.
@@ -677,7 +776,7 @@ async def async_process_component_config(
     domain = integration.domain
     try:
         component = integration.get_component()
-    except ImportError as ex:
+    except LOAD_EXCEPTIONS as ex:
         _LOGGER.error("Unable to import %s: %s", domain, ex)
         return None
 
@@ -685,8 +784,14 @@ async def async_process_component_config(
     config_validator = None
     try:
         config_validator = integration.get_platform("config")
-    except ImportError:
-        pass
+    except ImportError as err:
+        # Filter out import error of the config platform.
+        # If the config platform contains bad imports, make sure
+        # that still fails.
+        if err.name != f"{integration.pkg_path}.config":
+            _LOGGER.error("Error importing config platform %s: %s", domain, err)
+            return None
+
     if config_validator is not None and hasattr(
         config_validator, "async_validate_config"
     ):
@@ -750,16 +855,14 @@ async def async_process_component_config(
 
         try:
             platform = p_integration.get_platform(domain)
-        except ImportError:
+        except LOAD_EXCEPTIONS:
             _LOGGER.exception("Platform error: %s", domain)
             continue
 
         # Validate platform specific schema
         if hasattr(platform, "PLATFORM_SCHEMA"):
             try:
-                p_validated = platform.PLATFORM_SCHEMA(  # type: ignore
-                    p_config
-                )
+                p_validated = platform.PLATFORM_SCHEMA(p_config)  # type: ignore
             except vol.Invalid as ex:
                 async_log_exception(
                     ex,
@@ -788,18 +891,19 @@ async def async_process_component_config(
 
 
 @callback
-def config_without_domain(config: Dict, domain: str) -> Dict:
+def config_without_domain(config: dict, domain: str) -> dict:
     """Return a config with all configuration for a domain removed."""
     filter_keys = extract_domain_configs(config, domain)
     return {key: value for key, value in config.items() if key not in filter_keys}
 
 
-async def async_check_ha_config_file(hass: HomeAssistant) -> Optional[str]:
+async def async_check_ha_config_file(hass: HomeAssistant) -> str | None:
     """Check if Home Assistant configuration file is valid.
 
     This method is a coroutine.
     """
-    import homeassistant.helpers.check_config as check_config
+    # pylint: disable=import-outside-toplevel
+    from homeassistant.helpers import check_config
 
     res = await check_config.async_check_ha_config_file(hass)
 
@@ -810,12 +914,13 @@ async def async_check_ha_config_file(hass: HomeAssistant) -> Optional[str]:
 
 @callback
 def async_notify_setup_error(
-    hass: HomeAssistant, component: str, display_link: Optional[str] = None
+    hass: HomeAssistant, component: str, display_link: str | None = None
 ) -> None:
     """Print a persistent notification.
 
     This method must be run in the event loop.
     """
+    # pylint: disable=import-outside-toplevel
     from homeassistant.components import persistent_notification
 
     errors = hass.data.get(DATA_PERSISTENT_ERRORS)
@@ -828,14 +933,10 @@ def async_notify_setup_error(
     message = "The following integrations and platforms could not be set up:\n\n"
 
     for name, link in errors.items():
-        if link:
-            part = f"[{name}]({link})"
-        else:
-            part = name
-
+        part = f"[{name}]({link})" if link else name
         message += f" - {part}\n"
 
-    message += "\nPlease check your config."
+    message += "\nPlease check your config and [logs](/config/logs)."
 
     persistent_notification.async_create(
         hass, message, "Invalid config", "invalid_config"
