@@ -4,8 +4,8 @@ from __future__ import annotations
 from typing import Final
 
 MAJOR_VERSION: Final = 2021
-MINOR_VERSION: Final = 7
-PATCH_VERSION: Final = "3"
+MINOR_VERSION: Final = 8
+PATCH_VERSION: Final = "0"
 __short_version__: Final = f"{MAJOR_VERSION}.{MINOR_VERSION}"
 __version__: Final = f"{__short_version__}.{PATCH_VERSION}"
 REQUIRED_PYTHON_VER: Final[tuple[int, int, int]] = (3, 8, 0)
@@ -266,6 +266,7 @@ STATE_ALARM_DISARMED: Final = "disarmed"
 STATE_ALARM_ARMED_HOME: Final = "armed_home"
 STATE_ALARM_ARMED_AWAY: Final = "armed_away"
 STATE_ALARM_ARMED_NIGHT: Final = "armed_night"
+STATE_ALARM_ARMED_VACATION: Final = "armed_vacation"
 STATE_ALARM_ARMED_CUSTOM_BYPASS: Final = "armed_custom_bypass"
 STATE_ALARM_PENDING: Final = "pending"
 STATE_ALARM_ARMING: Final = "arming"
@@ -273,6 +274,9 @@ STATE_ALARM_DISARMING: Final = "disarming"
 STATE_ALARM_TRIGGERED: Final = "triggered"
 STATE_LOCKED: Final = "locked"
 STATE_UNLOCKED: Final = "unlocked"
+STATE_LOCKING: Final = "locking"
+STATE_UNLOCKING: Final = "unlocking"
+STATE_JAMMED: Final = "jammed"
 STATE_UNAVAILABLE: Final = "unavailable"
 STATE_OK: Final = "ok"
 STATE_PROBLEM: Final = "problem"
@@ -392,21 +396,24 @@ ATTR_DEVICE_CLASS: Final = "device_class"
 # Temperature attribute
 ATTR_TEMPERATURE: Final = "temperature"
 
+
 # #### UNITS OF MEASUREMENT ####
 # Power units
 POWER_WATT: Final = "W"
 POWER_KILO_WATT: Final = "kW"
-
-# Voltage units
-VOLT: Final = "V"
+POWER_VOLT_AMPERE: Final = "VA"
 
 # Energy units
 ENERGY_WATT_HOUR: Final = "Wh"
 ENERGY_KILO_WATT_HOUR: Final = "kWh"
 
-# Electrical units
-ELECTRICAL_CURRENT_AMPERE: Final = "A"
-ELECTRICAL_VOLT_AMPERE: Final = "VA"
+# Electric_current units
+ELECTRIC_CURRENT_MILLIAMPERE: Final = "mA"
+ELECTRIC_CURRENT_AMPERE: Final = "A"
+
+# Electric_potential units
+ELECTRIC_POTENTIAL_MILLIVOLT: Final = "mV"
+ELECTRIC_POTENTIAL_VOLT: Final = "V"
 
 # Degree units
 DEGREE: Final = "°"
@@ -445,6 +452,7 @@ LENGTH_MILES: Final = "mi"
 
 # Frequency units
 FREQUENCY_HERTZ: Final = "Hz"
+FREQUENCY_MEGAHERTZ: Final = "MHz"
 FREQUENCY_GIGAHERTZ: Final = "GHz"
 
 # Pressure units
@@ -455,6 +463,10 @@ PRESSURE_MBAR: Final = "mbar"
 PRESSURE_INHG: Final = "inHg"
 PRESSURE_PSI: Final = "psi"
 
+# Sound pressure units
+SOUND_PRESSURE_DB: Final = "dB"
+SOUND_PRESSURE_WEIGHTED_DBA: Final = "dBa"
+
 # Volume units
 VOLUME_LITERS: Final = "L"
 VOLUME_MILLILITERS: Final = "mL"
@@ -498,6 +510,8 @@ IRRADIATION_BTUS_PER_HOUR_SQUARE_FOOT: Final = "BTU/(h×ft²)"
 
 # Precipitation units
 PRECIPITATION_MILLIMETERS_PER_HOUR: Final = "mm/h"
+PRECIPITATION_INCHES: Final = "in"
+PRECIPITATION_INCHES_PER_HOUR: Final = "in/h"
 
 # Concentration units
 CONCENTRATION_MICROGRAMS_PER_CUBIC_METER: Final = "µg/m³"
@@ -541,6 +555,8 @@ DATA_PEBIBYTES: Final = "PiB"
 DATA_EXBIBYTES: Final = "EiB"
 DATA_ZEBIBYTES: Final = "ZiB"
 DATA_YOBIBYTES: Final = "YiB"
+
+# Data_rate units
 DATA_RATE_BITS_PER_SECOND: Final = "bit/s"
 DATA_RATE_KILOBITS_PER_SECOND: Final = "kbit/s"
 DATA_RATE_MEGABITS_PER_SECOND: Final = "Mbit/s"
@@ -553,6 +569,7 @@ DATA_RATE_KIBIBYTES_PER_SECOND: Final = "KiB/s"
 DATA_RATE_MEBIBYTES_PER_SECOND: Final = "MiB/s"
 DATA_RATE_GIBIBYTES_PER_SECOND: Final = "GiB/s"
 
+
 # #### SERVICES ####
 SERVICE_HOMEASSISTANT_STOP: Final = "stop"
 SERVICE_HOMEASSISTANT_RESTART: Final = "restart"
@@ -580,6 +597,7 @@ SERVICE_ALARM_DISARM: Final = "alarm_disarm"
 SERVICE_ALARM_ARM_HOME: Final = "alarm_arm_home"
 SERVICE_ALARM_ARM_AWAY: Final = "alarm_arm_away"
 SERVICE_ALARM_ARM_NIGHT: Final = "alarm_arm_night"
+SERVICE_ALARM_ARM_VACATION: Final = "alarm_arm_vacation"
 SERVICE_ALARM_ARM_CUSTOM_BYPASS: Final = "alarm_arm_custom_bypass"
 SERVICE_ALARM_TRIGGER: Final = "alarm_trigger"
 
